@@ -18,8 +18,6 @@
 # MA 02110-1301 USA.
 #
 
-from gi.repository import GObject
-
 import logging
 import os
 import threading
@@ -27,6 +25,9 @@ import time
 import traceback
 
 import libvirt
+
+from gi.repository import GObject
+
 import virtinst
 from virtinst import pollhelpers
 from virtinst import support
@@ -42,17 +43,22 @@ from .storagepool import vmmStoragePool
 
 
 # debugging helper to turn off events
-_disable_libvirt_events = False
+# Can be enabled with virt-manager --test-no-events
+FORCE_DISABLE_EVENTS = False
 
 
 class _ObjectList(vmmGObject):
     """
     Class that wraps our internal list of libvirt objects
     """
+
+    BLACKLIST_COUNT = 3
+
     def __init__(self):
         vmmGObject.__init__(self)
 
         self._objects = []
+        self._blacklist = {}
         self._lock = threading.Lock()
 
     def _cleanup(self):
@@ -62,12 +68,46 @@ class _ObjectList(vmmGObject):
             for obj in self._objects:
                 try:
                     obj.cleanup()
-                except:
+                except Exception:
                     logging.debug("Failed to cleanup %s", exc_info=True)
             self._objects = []
         finally:
             self._lock.release()
 
+    def _blacklist_key(self, obj):
+        return str(obj.__class__) + obj.get_connkey()
+
+    def add_blacklist(self, obj):
+        """
+        Add an object to the blacklist. Basically a list of objects we
+        choose not to poll, because they threw an error at init time
+
+        :param obj: vmmLibvirtObject to blacklist
+        :returns: number of added object to list
+        """
+        key = self._blacklist_key(obj)
+        if self.in_blacklist(obj):
+            self._blacklist[key] += 1
+        self._blacklist[key] = 1
+        return self._blacklist[key]
+
+    def remove_blacklist(self, obj):
+        """
+        :param obj: vmmLibvirtObject to remove from blacklist
+        :returns: True if object was blacklisted or False otherwise.
+        """
+        return bool(self._blacklist.pop(self._blacklist_key(obj), 0))
+
+    def in_blacklist(self, obj):
+        """
+        If an object is in list only once don't consider it blacklisted,
+        give it one more chance.
+
+        :param obj: vmmLibvirtObject to check
+        :returns: True if object is blacklisted
+        """
+        return self._blacklist.get(self._blacklist_key(obj), 0) > _ObjectList.BLACKLIST_COUNT
+
     def remove(self, obj):
         """
         Remove an object from the list.
@@ -81,7 +121,7 @@ class _ObjectList(vmmGObject):
             # Identity check is sufficient here, since we should never be
             # asked to remove an object that wasn't at one point in the list.
             if obj not in self._objects:
-                return False
+                return self.remove_blacklist(obj)
 
             self._objects.remove(obj)
             return True
@@ -140,6 +180,7 @@ class vmmConnection(vmmGObject):
     __gsignals__ = {
         "vm-added": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "vm-removed": (GObject.SignalFlags.RUN_FIRST, None, [str]),
+        "vm-renamed": (GObject.SignalFlags.RUN_FIRST, None, [str, str]),
         "net-added": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "net-removed": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "pool-added": (GObject.SignalFlags.RUN_FIRST, None, [str]),
@@ -182,6 +223,10 @@ class vmmConnection(vmmGObject):
         self._domain_cb_ids = []
         self.using_network_events = False
         self._network_cb_ids = []
+        self.using_storage_pool_events = False
+        self._storage_pool_cb_ids = []
+        self.using_node_device_events = False
+        self._node_device_cb_ids = []
 
         self._xml_flags = {}
 
@@ -218,6 +263,10 @@ class vmmConnection(vmmGObject):
                 label = "test (xen)"
             elif gtype == "hvm":
                 label = "test (hvm)"
+        elif domtype == "qemu":
+            label = "QEMU TCG"
+        elif domtype == "kvm":
+            label = "KVM"
 
         return label
 
@@ -225,6 +274,29 @@ class vmmConnection(vmmGObject):
     # Init routines #
     #################
 
+    def _wait_for_condition(self, compare_cb, timeout=3):
+        """
+        Wait for this object to emit the specified signal. Will not
+        block the mainloop.
+        """
+        from gi.repository import Gtk
+        is_main_thread = (threading.current_thread().name == "MainThread")
+        start_time = time.time()
+
+        while True:
+            cur_time = time.time()
+            if compare_cb():
+                return
+            if (cur_time - start_time) >= timeout:
+                return
+
+            if is_main_thread:
+                if Gtk.events_pending():
+                    Gtk.main_iteration_do(False)
+                    continue
+
+            time.sleep(.1)
+
     def _init_virtconn(self):
         self._backend.cb_fetch_all_guests = (
             lambda: [obj.get_xmlobj(refresh_if_nec=False)
@@ -242,20 +314,20 @@ class vmmConnection(vmmGObject):
                 for vol in pool.get_volumes():
                     try:
                         ret.append(vol.get_xmlobj(refresh_if_nec=False))
-                    except Exception, e:
+                    except Exception as e:
                         logging.debug("Fetching volume XML failed: %s", e)
             return ret
         self._backend.cb_fetch_all_vols = fetch_all_vols
 
-        def clear_cache(pools=False):
-            if not pools:
+        def cache_new_pool(obj):
+            if not self.is_active():
                 return
-
-            # This isn't synchronous, so any virtinst callers need to
-            # take that into account.
+            name = obj.name()
             self.schedule_priority_tick(pollpool=True)
-
-        self._backend.cb_clear_cache = clear_cache
+            def compare_cb():
+                return bool(self.get_pool(name))
+            self._wait_for_condition(compare_cb)
+        self._backend.cb_cache_new_pool = cache_new_pool
 
 
     ########################
@@ -309,13 +381,14 @@ class vmmConnection(vmmGObject):
     is_container = property(lambda s: getattr(s, "_backend").is_container)
     is_lxc = property(lambda s: getattr(s, "_backend").is_lxc)
     is_openvz = property(lambda s: getattr(s, "_backend").is_openvz)
+    is_vz = property(lambda s: getattr(s, "_backend").is_vz)
     is_xen = property(lambda s: getattr(s, "_backend").is_xen)
     is_remote = property(lambda s: getattr(s, "_backend").is_remote)
     is_qemu = property(lambda s: getattr(s, "_backend").is_qemu)
     is_qemu_system = property(lambda s: getattr(s, "_backend").is_qemu_system)
     is_qemu_session = property(lambda s:
                                getattr(s, "_backend").is_qemu_session)
-    is_test_conn = property(lambda s: getattr(s, "_backend").is_test)
+    is_test = property(lambda s: getattr(s, "_backend").is_test)
     is_session_uri = property(lambda s: getattr(s, "_backend").is_session_uri)
 
 
@@ -327,7 +400,7 @@ class vmmConnection(vmmGObject):
         uri = self.get_uri().replace("/", "_")
         ret = os.path.join(util.get_cache_dir(), uri)
         if not os.path.exists(ret):
-            os.makedirs(ret, 0755)
+            os.makedirs(ret, 0o755)
         return ret
 
     def get_default_storage_format(self):
@@ -355,23 +428,21 @@ class vmmConnection(vmmGObject):
         """
         if self._get_config_pretty_name():
             return self._get_config_pretty_name()
-        if self._backend.fake_name():
-            return self._backend.fake_name()
 
         pretty_map = {
-            "esx"       : "ESX",
-            "gsx"       : "GSX",
-            "libxl"     : "libxl",
-            "lxc"       : "LXC",
-            "openvz"    : "OpenVZ",
-            "phyp"      : "phyp",
-            "qemu"      : "QEMU/KVM",
-            "test"      : "test",
-            "uml"       : "UML",
-            "vbox"      : "VBox",
-            "vmware"    : "VMWare",
-            "xen"       : "xen",
-            "xenapi"    : "XenAPI",
+            "esx":        "ESX",
+            "gsx":        "GSX",
+            "libxl":      "libxl",
+            "lxc":        "LXC",
+            "openvz":     "OpenVZ",
+            "phyp":       "phyp",
+            "qemu":       "QEMU/KVM",
+            "test":       "test",
+            "uml":        "UML",
+            "vbox":       "VBox",
+            "vmware":     "VMWare",
+            "xen":        "Xen",
+            "xenapi":     "XenAPI",
         }
 
         hv = pretty_map.get(self.get_driver(), self.get_driver())
@@ -382,8 +453,8 @@ class vmmConnection(vmmGObject):
         ret = hv
 
         if is_session:
-            ret += " User session"
-        elif path != "/system":
+            ret += " " + _("User session")
+        elif (path and path != "/system" and os.path.basename(path)):
             # Used by test URIs to report what XML file they are using
             ret += " %s" % os.path.basename(path)
 
@@ -403,6 +474,7 @@ class vmmConnection(vmmGObject):
         locals()[_supportname] = getattr(virtinst.VirtualConnection,
                                          _supportname)
     def check_support(self, *args):
+        # pylint: disable=no-value-for-parameter
         return self._backend.check_support(*args)
 
     def is_storage_capable(self):
@@ -412,12 +484,6 @@ class vmmConnection(vmmGObject):
             if self._storage_capable is False:
                 logging.debug("Connection doesn't seem to support storage "
                               "APIs. Skipping all storage polling.")
-            else:
-                # Try to create the default storage pool
-                try:
-                    virtinst.StoragePool.build_default_pool(self.get_backend())
-                except Exception, e:
-                    logging.debug("Building default pool failed: %s", str(e))
 
         return self._storage_capable
 
@@ -518,7 +584,7 @@ class vmmConnection(vmmGObject):
                 try:
                     if vol.get_target_path() == path:
                         return vol
-                except Exception, e:
+                except Exception as e:
                     # Errors can happen if the volume disappeared, bug 1092739
                     logging.debug("Error looking up volume from path=%s: %s",
                         path, e)
@@ -593,10 +659,12 @@ class vmmConnection(vmmGObject):
         for dev in self.list_nodedevs():
             try:
                 xmlobj = dev.get_xmlobj()
-            except libvirt.libvirtError, e:
-                if e.get_error_code() == libvirt.VIR_ERR_NO_NODE_DEVICE:
-                    continue
-                raise
+            except libvirt.libvirtError as e:
+                # Libvirt nodedev XML fetching can be busted
+                # https://bugzilla.redhat.com/show_bug.cgi?id=1225771
+                if e.get_error_code() != libvirt.VIR_ERR_NO_NODE_DEVICE:
+                    logging.debug("Error fetching nodedev XML", exc_info=True)
+                continue
 
             if devtype and xmlobj.device_type != devtype:
                 continue
@@ -648,10 +716,7 @@ class vmmConnection(vmmGObject):
     def define_interface(self, xml):
         return self._backend.interfaceDefineXML(xml, 0)
 
-    def rename_object(self, obj, origxml, newxml, oldname, newname):
-        ignore = oldname
-        ignore = newname
-
+    def rename_object(self, obj, origxml, newxml, oldconnkey):
         if obj.class_name() == "domain":
             define_cb = self.define_domain
         elif obj.class_name() == "pool":
@@ -669,12 +734,12 @@ class vmmConnection(vmmGObject):
         try:
             # Redefine new domain
             newobj = define_cb(newxml)
-        except Exception, renameerr:
+        except Exception as renameerr:
             try:
                 logging.debug("Error defining new name %s XML",
                     obj.class_name(), exc_info=True)
                 newobj = define_cb(origxml)
-            except Exception, fixerr:
+            except Exception as fixerr:
                 logging.debug("Failed to redefine original %s!",
                     obj.class_name(), exc_info=True)
                 raise RuntimeError(
@@ -689,6 +754,9 @@ class vmmConnection(vmmGObject):
                 # Reinsert handle into new obj
                 obj.change_name_backend(newobj)
 
+        if newobj and obj.class_name() == "domain":
+            self.emit("vm-renamed", oldconnkey, obj.get_connkey())
+
 
     #########################
     # Domain event handling #
@@ -709,7 +777,7 @@ class vmmConnection(vmmGObject):
         if not obj:
             return
 
-        self.idle_add(obj.refresh_from_event_loop)
+        self.idle_add(obj.recache_from_event_loop)
 
     def _domain_lifecycle_event(self, conn, domain, event, reason, userdata):
         ignore = conn
@@ -721,7 +789,7 @@ class vmmConnection(vmmGObject):
         obj = self.get_vm(name)
 
         if obj:
-            self.idle_add(obj.refresh_from_event_loop)
+            self.idle_add(obj.recache_from_event_loop)
         else:
             self.schedule_priority_tick(pollvm=True, force=True)
 
@@ -735,17 +803,70 @@ class vmmConnection(vmmGObject):
         obj = self.get_net(name)
 
         if obj:
-            self.idle_add(obj.refresh_from_event_loop)
+            self.idle_add(obj.recache_from_event_loop)
         else:
             self.schedule_priority_tick(pollnet=True, force=True)
 
+    def _storage_pool_lifecycle_event(self, conn, pool,
+                                      event, reason, userdata):
+        ignore = conn
+        ignore = userdata
+
+        name = pool.name()
+        logging.debug("storage pool lifecycle event: storage=%s event=%s "
+            "reason=%s", name, event, reason)
+
+        obj = self.get_pool(name)
+
+        if obj:
+            self.idle_add(obj.recache_from_event_loop)
+        else:
+            self.schedule_priority_tick(pollpool=True, force=True)
+
+    def _storage_pool_refresh_event(self, conn, pool, userdata):
+        ignore = conn
+        ignore = userdata
+
+        name = pool.name()
+        logging.debug("storage pool refresh event: pool=%s", name)
+
+        obj = self.get_pool(name)
+
+        if not obj:
+            return
+
+        self.idle_add(obj.refresh_pool_cache_from_event_loop)
+
+    def _node_device_lifecycle_event(self, conn, dev,
+                                     event, reason, userdata):
+        ignore = conn
+        ignore = userdata
+
+        name = dev.name()
+        logging.debug("node device lifecycle event: device=%s event=%s "
+            "reason=%s", name, event, reason)
+
+        self.schedule_priority_tick(pollnodedev=True, force=True)
+
+    def _node_device_update_event(self, conn, dev, userdata):
+        ignore = conn
+        ignore = userdata
+
+        name = dev.name()
+        logging.debug("node device update event: device=%s", name)
+
+        obj = self.get_nodedev(name)
+
+        if obj:
+            self.idle_add(obj.recache_from_event_loop)
+
     def _add_conn_events(self):
         if not self.check_support(support.SUPPORT_CONN_WORKING_XEN_EVENTS):
             return
 
         try:
-            if _disable_libvirt_events:
-                raise RuntimeError("_disable_libvirt_events = True")
+            if FORCE_DISABLE_EVENTS:
+                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
 
             self._domain_cb_ids.append(
                 self.get_backend().domainEventRegisterAny(
@@ -753,7 +874,7 @@ class vmmConnection(vmmGObject):
                 self._domain_lifecycle_event, None))
             self.using_domain_events = True
             logging.debug("Using domain events")
-        except Exception, e:
+        except Exception as e:
             self.using_domain_events = False
             logging.debug("Error registering domain events: %s", e)
 
@@ -764,7 +885,7 @@ class vmmConnection(vmmGObject):
                 self._domain_cb_ids.append(
                     self.get_backend().domainEventRegisterAny(
                     None, eventid, self._domain_xml_misc_event, None))
-            except Exception, e:
+            except Exception as e:
                 logging.debug("Error registering domain %s event: %s",
                     typestr, e)
 
@@ -781,8 +902,8 @@ class vmmConnection(vmmGObject):
             "device added")
 
         try:
-            if _disable_libvirt_events:
-                raise RuntimeError("_disable_libvirt_events = True")
+            if FORCE_DISABLE_EVENTS:
+                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
 
             eventid = getattr(libvirt, "VIR_NETWORK_EVENT_ID_LIFECYCLE", 0)
             self._network_cb_ids.append(
@@ -790,10 +911,49 @@ class vmmConnection(vmmGObject):
                 None, eventid, self._network_lifecycle_event, None))
             self.using_network_events = True
             logging.debug("Using network events")
-        except Exception, e:
+        except Exception as e:
             self.using_network_events = False
             logging.debug("Error registering network events: %s", e)
 
+        try:
+            if FORCE_DISABLE_EVENTS:
+                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
+
+            eventid = getattr(libvirt,
+                              "VIR_STORAGE_POOL_EVENT_ID_LIFECYCLE", 0)
+            refreshid = getattr(libvirt,
+                              "VIR_STORAGE_POOL_EVENT_ID_REFRESH", 1)
+            self._storage_pool_cb_ids.append(
+                self.get_backend().storagePoolEventRegisterAny(
+                None, eventid, self._storage_pool_lifecycle_event, None))
+            self._storage_pool_cb_ids.append(
+                self.get_backend().storagePoolEventRegisterAny(
+                None, refreshid, self._storage_pool_refresh_event, None))
+            self.using_storage_pool_events = True
+            logging.debug("Using storage pool events")
+        except Exception as e:
+            self.using_storage_pool_events = False
+            logging.debug("Error registering storage pool events: %s", e)
+
+        try:
+            if FORCE_DISABLE_EVENTS:
+                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
+
+            eventid = getattr(libvirt, "VIR_NODE_DEVICE_EVENT_ID_LIFECYCLE", 0)
+            updateid = getattr(libvirt, "VIR_NODE_DEVICE_EVENT_ID_UPDATE", 1)
+            self._node_device_cb_ids.append(
+                self.get_backend().nodeDeviceEventRegisterAny(
+                None, eventid, self._node_device_lifecycle_event, None))
+            self._node_device_cb_ids.append(
+                self.get_backend().nodeDeviceEventRegisterAny(
+                None, updateid, self._node_device_update_event, None))
+
+            self.using_node_device_events = True
+            logging.debug("Using node device events")
+        except Exception as e:
+            self.using_network_events = False
+            logging.debug("Error registering node device events: %s", e)
+
 
     ######################################
     # Connection closing/opening methods #
@@ -814,12 +974,18 @@ class vmmConnection(vmmGObject):
                     self._backend.domainEventDeregisterAny(eid)
                 for eid in self._network_cb_ids:
                     self._backend.networkEventDeregisterAny(eid)
-        except:
+                for eid in self._storage_pool_cb_ids:
+                    self._backend.storagePoolEventDeregisterAny(eid)
+                for eid in self._node_device_cb_ids:
+                    self._backend.nodeDeviceEventDeregisterAny(eid)
+        except Exception:
             logging.debug("Failed to deregister events in conn cleanup",
                 exc_info=True)
         finally:
             self._domain_cb_ids = []
             self._network_cb_ids = []
+            self._storage_pool_cb_ids = []
+            self._node_device_cb_ids = []
 
         self._backend.close()
         self._stats = []
@@ -836,6 +1002,13 @@ class vmmConnection(vmmGObject):
     def _cleanup(self):
         self.close()
 
+        self._objects = None
+        self._backend.cb_fetch_all_guests = None
+        self._backend.cb_fetch_all_pools = None
+        self._backend.cb_fetch_all_nodedevs = None
+        self._backend.cb_fetch_all_vols = None
+        self._backend.cb_cache_new_pool = None
+
     def open(self):
         if not self.is_disconnected():
             return
@@ -849,7 +1022,7 @@ class vmmConnection(vmmGObject):
     def _do_creds_password(self, creds):
         try:
             return connectauth.creds_dialog(self, creds)
-        except:
+        except Exception:
             logging.debug("Launching creds dialog failed", exc_info=True)
             return -1
 
@@ -861,9 +1034,10 @@ class vmmConnection(vmmGObject):
         try:
             self._backend.open(self._do_creds_password)
             return True, None
-        except Exception, exc:
+        except Exception as exc:
             tb = "".join(traceback.format_exc())
-            if type(exc) is libvirt.libvirtError:
+            if isinstance(exc, libvirt.libvirtError):
+                # pylint: disable=no-member
                 libvirt_error_code = exc.get_error_code()
                 libvirt_error_message = exc.get_error_message()
 
@@ -898,14 +1072,22 @@ class vmmConnection(vmmGObject):
         logging.debug("conn version=%s", self._backend.conn_version())
         logging.debug("%s capabilities:\n%s",
                       self.get_uri(), self.caps.get_xml_config())
+
+        # Try to create the default storage pool
+        # We want this before events setup to save some needless polling
+        try:
+            virtinst.StoragePool.build_default_pool(self.get_backend())
+        except Exception as e:
+            logging.debug("Building default pool failed: %s", str(e))
+
         self._add_conn_events()
 
         # Prime CPU cache
-        self.caps.get_cpu_values("x86_64")
+        self.caps.get_cpu_values(self.caps.host.cpu.arch)
 
         try:
             self._backend.setKeepAlive(20, 1)
-        except Exception, e:
+        except Exception as e:
             if (type(e) is not AttributeError and
                 not util.is_error_nosupport(e)):
                 raise
@@ -937,7 +1119,7 @@ class vmmConnection(vmmGObject):
 
             self.idle_add(self._change_state, is_active and
                 self._STATE_ACTIVE or self._STATE_DISCONNECTED)
-        except Exception, e:
+        except Exception as e:
             is_active = False
             self._schedule_close()
             connectError = (str(e), "".join(traceback.format_exc()), False)
@@ -964,7 +1146,7 @@ class vmmConnection(vmmGObject):
             class_name = obj.class_name()
             try:
                 name = obj.get_name()
-            except:
+            except Exception:
                 name = str(obj)
 
             if not self._objects.remove(obj):
@@ -985,13 +1167,24 @@ class vmmConnection(vmmGObject):
                 self.emit("nodedev-removed", obj.get_connkey())
             obj.cleanup()
 
-    def _new_object_cb(self, obj):
+    def _new_object_cb(self, obj, initialize_failed, skip_init=False):
         if not self._backend.is_open():
             return
 
         try:
             class_name = obj.class_name()
 
+            if initialize_failed:
+                logging.debug("Blacklisting %s=%s", class_name, obj.get_name())
+                count = self._objects.add_blacklist(obj)
+                if count <= _ObjectList.BLACKLIST_COUNT:
+                    logging.debug("Object added in blacklist, count=%d", count)
+                else:
+                    logging.debug("Object already blacklisted?")
+                return
+            else:
+                self._objects.remove_blacklist(obj)
+
             if not self._objects.add(obj):
                 logging.debug("New %s=%s requested, but it's already tracked.",
                     class_name, obj.get_name())
@@ -1012,7 +1205,7 @@ class vmmConnection(vmmGObject):
             elif class_name == "nodedev":
                 self.emit("nodedev-added", obj.get_connkey())
         finally:
-            if self._init_object_event:
+            if self._init_object_event and not skip_init:
                 self._init_object_count -= 1
                 if self._init_object_count <= 0:
                     self._init_object_event.set()
@@ -1069,6 +1262,7 @@ class vmmConnection(vmmGObject):
 
             gone_objects.extend(gone)
             preexisting_objects.extend([o for o in master if o not in new])
+            new = [n for n in new if not self._objects.in_blacklist(n)]
             return new
 
         new_vms = _process_objects(self._update_vms(pollvm))
@@ -1127,6 +1321,10 @@ class vmmConnection(vmmGObject):
             pollvm = False
         if self.using_network_events and not force:
             pollnet = False
+        if self.using_storage_pool_events and not force:
+            pollpool = False
+        if self.using_node_device_events and not force:
+            pollnodedev = False
 
         self._hostinfo = self._backend.getInfo()
 
@@ -1152,7 +1350,7 @@ class vmmConnection(vmmGObject):
                     continue
 
                 obj.tick(stats_update=stats_update)
-            except Exception, e:
+            except Exception as e:
                 logging.exception("Tick for %s failed", obj)
                 if (isinstance(e, libvirt.libvirtError) and
                     (getattr(e, "get_error_code")() ==
@@ -1221,12 +1419,12 @@ class vmmConnection(vmmGObject):
             "memoryPercent": pcentMem,
             "cpuTime": cpuTime,
             "cpuHostPercent": pcentHostCpu,
-            "diskRdRate" : rdRate,
-            "diskWrRate" : wrRate,
-            "netRxRate" : rxRate,
-            "netTxRate" : txRate,
-            "diskMaxRate" : diskMaxRate,
-            "netMaxRate" : netMaxRate,
+            "diskRdRate": rdRate,
+            "diskWrRate": wrRate,
+            "netRxRate": rxRate,
+            "netTxRate": txRate,
+            "diskMaxRate": diskMaxRate,
+            "netMaxRate": netMaxRate,
         }
 
         self._stats.insert(0, newStats)
@@ -1241,7 +1439,7 @@ class vmmConnection(vmmGObject):
             self._tick(*args, **kwargs)
         except KeyboardInterrupt:
             raise
-        except Exception, e:
+        except Exception as e:
             pass
 
         if e is None:
@@ -1250,10 +1448,12 @@ class vmmConnection(vmmGObject):
         from_remote = getattr(libvirt, "VIR_FROM_REMOTE", None)
         from_rpc = getattr(libvirt, "VIR_FROM_RPC", None)
         sys_error = getattr(libvirt, "VIR_ERR_SYSTEM_ERROR", None)
+        internal_error = getattr(libvirt, "VIR_ERR_INTERNAL_ERROR", None)
 
         dom = -1
         code = -1
         if isinstance(e, libvirt.libvirtError):
+            # pylint: disable=no-member
             dom = e.get_error_domain()
             code = e.get_error_code()
 
@@ -1261,7 +1461,7 @@ class vmmConnection(vmmGObject):
             self.get_uri(), exc_info=True)
 
         if (dom in [from_remote, from_rpc] and
-            code in [sys_error]):
+            code in [sys_error, internal_error]):
             e = None
             logging.debug("Not showing user error since libvirtd "
                 "appears to have stopped.")
@@ -1328,8 +1528,14 @@ class vmmConnection(vmmGObject):
         self.config.set_conn_autoconnect(self.get_uri(), val)
 
     def set_config_pretty_name(self, value):
-        if value != self._get_config_pretty_name():
-            self.config.set_perconn(self.get_uri(), "/pretty-name", value)
+        cfgname = self._get_config_pretty_name()
+        if value == cfgname:
+            return
+        if not cfgname and value == self.get_pretty_desc():
+            # Don't encode the default connection value into gconf right
+            # away, require the user to edit it first
+            return
+        self.config.set_perconn(self.get_uri(), "/pretty-name", value)
     def _get_config_pretty_name(self):
         return self.config.get_perconn(self.get_uri(), "/pretty-name")
     def _on_config_pretty_name_changed(self, *args, **kwargs):
