@@ -19,30 +19,107 @@
 # MA 02110-1301 USA.
 #
 
+import logging
+
 from gi.repository import Gtk
 from gi.repository import Gdk
 
-import logging
-
-from .autodrawer import AutoDrawer
-from .baseclass import vmmGObjectUI
+from .baseclass import vmmGObject, vmmGObjectUI
 from .details import DETAILS_PAGE_CONSOLE
 from .serialcon import vmmSerialConsole
 from .sshtunnels import ConnectionInfo
-from .viewers import SpiceViewer, VNCViewer
+from .viewers import SpiceViewer, VNCViewer, have_spice_gtk
+
+
+# console-pages IDs
+(_CONSOLE_PAGE_UNAVAILABLE,
+ _CONSOLE_PAGE_AUTHENTICATE,
+ _CONSOLE_PAGE_SERIAL,
+ _CONSOLE_PAGE_VIEWER) = range(4)
+
+
+class _TimedRevealer(vmmGObject):
+    """
+    Revealer for the fullscreen toolbar, with a bit of extra logic to
+    hide/show based on mouse over
+    """
+    def __init__(self, toolbar):
+        vmmGObject.__init__(self)
+
+        self._in_fullscreen = False
+        self._timeout_id = None
+
+        self._revealer = Gtk.Revealer()
+        self._revealer.add(toolbar)
+
+        # Adding the revealer to the eventbox seems to ensure the
+        # eventbox always has 1 invisible pixel showing at the top of the
+        # screen, which we can use to grab the pointer event to show
+        # the hidden toolbar.
+
+        self._ebox = Gtk.EventBox()
+        self._ebox.add(self._revealer)
+        self._ebox.set_halign(Gtk.Align.CENTER)
+        self._ebox.set_valign(Gtk.Align.START)
+        self._ebox.show_all()
+
+        self._ebox.connect("enter-notify-event", self._enter_notify)
+        self._ebox.connect("leave-notify-event", self._enter_notify)
+
+    def _cleanup(self):
+        self._ebox.destroy()
+        self._ebox = None
+        self._revealer.destroy()
+        self._revealer = None
+        self._timeout_id = None
+
+    def _enter_notify(self, ignore1, ignore2):
+        x, y = self._ebox.get_pointer()
+        alloc = self._ebox.get_allocation()
+        entered = bool(x >= 0 and y >= 0 and
+                       x < alloc.width and y < alloc.height)
+
+        if not self._in_fullscreen:
+            return
+
+        # Pointer exited the toolbar, and toolbar is revealed. Schedule
+        # a timeout to close it, if one isn't already scheduled
+        if not entered and self._revealer.get_reveal_child():
+            self._schedule_unreveal_timeout(1000)
+            return
+
+        self._unregister_timeout()
+        if entered and not self._revealer.get_reveal_child():
+            self._revealer.set_reveal_child(True)
+
+    def _schedule_unreveal_timeout(self, timeout):
+        if self._timeout_id:
+            return
+
+        def cb():
+            self._revealer.set_reveal_child(False)
+            self._timeout_id = None
+        self._timeout_id = self.timeout_add(timeout, cb)
+
+    def _unregister_timeout(self):
+        if self._timeout_id:
+            self.remove_gobject_timeout(self._timeout_id)
+            self._timeout_id = None
+
+    def force_reveal(self, val):
+        self._unregister_timeout()
+        self._in_fullscreen = val
+        self._revealer.set_reveal_child(val)
+        self._schedule_unreveal_timeout(2000)
+
+    def get_overlay_widget(self):
+        return self._ebox
 
 
 class vmmConsolePages(vmmGObjectUI):
     """
     Handles all the complex UI handling dictated by the spice/vnc widgets
     """
-    # Console pages
-    (CONSOLE_PAGE_UNAVAILABLE,
-     CONSOLE_PAGE_AUTHENTICATE,
-     CONSOLE_PAGE_VIEWER,
-     CONSOLE_PAGE_OFFSET) = range(4)
-
-
     def __init__(self, vm, builder, topwin):
         vmmGObjectUI.__init__(self, None, None, builder=builder, topwin=topwin)
 
@@ -50,7 +127,6 @@ class vmmConsolePages(vmmGObjectUI):
         self._pointer_is_grabbed = False
         self._change_title()
         self.vm.connect("state-changed", self._change_title)
-        self._force_resize = False
 
         # State for disabling modifiers when keyboard is grabbed
         self._accel_groups = Gtk.accel_groups_from_object(self.topwin)
@@ -59,29 +135,35 @@ class vmmConsolePages(vmmGObjectUI):
 
         # Initialize display widget
         self._viewer = None
-        self._viewer_is_connected = False
 
         # Fullscreen toolbar
         self._send_key_button = None
-        self._fs_toolbar = None
-        self._fs_drawer = None
-        self._keycombo_menu = self._build_keycombo_menu(self._do_send_key)
-        self._init_fs_toolbar()
+        self._overlay_toolbar = None
+        self._timed_revealer = None
+        self._keycombo_toolbar = self._build_keycombo_menu()
+        self._keycombo_menu = self._build_keycombo_menu()
+        self._init_overlay_toolbar()
 
         # Make viewer widget background always be black
         black = Gdk.Color(0, 0, 0)
         self.widget("console-gfx-viewport").modify_bg(Gtk.StateType.NORMAL,
                                                       black)
 
-        self._serial_tabs = []
-        self._last_gfx_page = 0
+        self.widget("console-pages").set_show_tabs(False)
+        self.widget("serial-pages").set_show_tabs(False)
+
+        self._serial_consoles = []
         self._init_menus()
 
         # Signals are added by vmmDetails. Don't use connect_signals here
         # or it changes will be overwritten
 
-        self._refresh_can_fullscreen()
+        self.widget("console-gfx-scroll").connect("size-allocate",
+            self._scroll_size_allocate)
+
+        self._refresh_widget_states()
         self._refresh_scaling_from_settings()
+
         self.add_gsettings_handle(
             self.vm.on_console_scaling_changed(
                 self._refresh_scaling_from_settings))
@@ -89,14 +171,9 @@ class vmmConsolePages(vmmGObjectUI):
         self.add_gsettings_handle(
             self.vm.on_console_resizeguest_changed(
                 self._refresh_resizeguest_from_settings))
-
-        scroll = self.widget("console-gfx-scroll")
-        scroll.connect("size-allocate", self._scroll_size_allocate)
         self.add_gsettings_handle(
             self.config.on_console_accels_changed(self._refresh_enable_accel))
 
-        self._page_changed()
-
 
     def is_visible(self):
         if self.topwin:
@@ -111,29 +188,30 @@ class vmmConsolePages(vmmGObjectUI):
             self._viewer.cleanup()
         self._viewer = None
 
-        self._keycombo_menu.destroy()
-        self._keycombo_menu = None
-        self._fs_drawer.destroy()
-        self._fs_drawer = None
-        self._fs_toolbar.destroy()
-        self._fs_toolbar = None
+        self._keycombo_toolbar.destroy()
+        self._keycombo_toolbar = None
+        self._overlay_toolbar.destroy()
+        self._overlay_toolbar = None
 
-        for serial in self._serial_tabs:
+        self._timed_revealer.cleanup()
+        self._timed_revealer = None
+
+        for serial in self._serial_consoles:
             serial.cleanup()
-        self._serial_tabs = []
+        self._serial_consoles = []
 
 
     ##########################
     # Initialization helpers #
     ##########################
 
-    def _build_keycombo_menu(self, cb):
+    def _build_keycombo_menu(self):
         # Shared with vmmDetails
         menu = Gtk.Menu()
 
         def make_item(name, combo):
             item = Gtk.MenuItem.new_with_mnemonic(name)
-            item.connect("activate", cb, combo)
+            item.connect("activate", self._do_send_key, combo)
 
             menu.add(item)
 
@@ -150,34 +228,35 @@ class vmmConsolePages(vmmGObjectUI):
         menu.show_all()
         return menu
 
-    def _init_fs_toolbar(self):
-        scroll = self.widget("console-gfx-scroll")
-        pages = self.widget("console-pages")
-        pages.remove(scroll)
-
-        self._fs_toolbar = Gtk.Toolbar()
-        self._fs_toolbar.set_show_arrow(False)
-        self._fs_toolbar.set_no_show_all(True)
-        self._fs_toolbar.set_style(Gtk.ToolbarStyle.BOTH_HORIZ)
+    def _init_overlay_toolbar(self):
+        self._overlay_toolbar = Gtk.Toolbar()
+        self._overlay_toolbar.set_show_arrow(False)
+        self._overlay_toolbar.set_style(Gtk.ToolbarStyle.BOTH_HORIZ)
 
         # Exit fullscreen button
         button = Gtk.ToolButton.new_from_stock(Gtk.STOCK_LEAVE_FULLSCREEN)
         button.set_tooltip_text(_("Leave fullscreen"))
         button.show()
-        self._fs_toolbar.add(button)
+        self._overlay_toolbar.add(button)
         button.connect("clicked", self._leave_fullscreen)
 
         def keycombo_menu_clicked(src):
             ignore = src
-            def menu_location(menu, toolbar):
-                ignore = menu
+            def menu_location(*args):
+                # Signature changed at some point.
+                #  f23+    : args = menu, x, y, toolbar
+                #  rhel7.3 : args = menu, toolbar
+                if len(args) == 4:
+                    toolbar = args[3]
+                else:
+                    toolbar = args[1]
+
                 ignore, x, y = toolbar.get_window().get_origin()
                 height = toolbar.get_window().get_height()
-
                 return x, y + height, True
 
-            self._keycombo_menu.popup(None, None, menu_location,
-                                     self._fs_toolbar, 0,
+            self._keycombo_toolbar.popup(None, None, menu_location,
+                                     self._overlay_toolbar, 0,
                                      Gtk.get_current_event_time())
 
         self._send_key_button = Gtk.ToolButton()
@@ -186,22 +265,11 @@ class vmmConsolePages(vmmGObjectUI):
         self._send_key_button.set_tooltip_text(_("Send key combination"))
         self._send_key_button.show_all()
         self._send_key_button.connect("clicked", keycombo_menu_clicked)
-        self._fs_toolbar.add(self._send_key_button)
+        self._overlay_toolbar.add(self._send_key_button)
 
-        self._fs_drawer = AutoDrawer()
-        self._fs_drawer.set_active(False)
-        self._fs_drawer.set_over(self._fs_toolbar)
-        self._fs_drawer.set_under(scroll)
-        self._fs_drawer.set_offset(-1)
-        self._fs_drawer.set_fill(False)
-        self._fs_drawer.set_overlap_pixels(1)
-        self._fs_drawer.set_nooverlap_pixels(0)
-        self._fs_drawer.period = 20
-        self._fs_drawer.step = .1
-
-        self._fs_drawer.show_all()
-
-        pages.add(self._fs_drawer)
+        self._timed_revealer = _TimedRevealer(self._overlay_toolbar)
+        self.widget("console-overlay").add_overlay(
+                self._timed_revealer.get_overlay_widget())
 
     def _init_menus(self):
         # Serial list menu
@@ -209,13 +277,19 @@ class vmmConsolePages(vmmGObjectUI):
         smenu.connect("show", self._populate_serial_menu)
         self.widget("details-menu-view-serial-list").set_submenu(smenu)
 
+        # Keycombo menu (ctrl+alt+del etc.)
+        self.widget("details-menu-send-key").set_submenu(self._keycombo_menu)
+
 
     #################
     # Internal APIs #
     #################
 
     def _change_title(self, ignore1=None):
-        title = self.vm.get_name_or_title() + " " + _("Virtual Machine")
+        title = (_("%(vm-name)s on %(connection-name)s") % {
+            "vm-name": self.vm.get_name_or_title(),
+            "connection-name": self.vm.conn.get_pretty_desc(),
+        })
 
         if self._pointer_is_grabbed and self._viewer:
             keystr = self._viewer.console_get_grab_keys()
@@ -228,10 +302,10 @@ class vmmConsolePages(vmmGObjectUI):
     def _someone_has_focus(self):
         if (self._viewer and
             self._viewer.console_has_focus() and
-            self._viewer_is_connected):
+            self._viewer.console_is_open()):
             return True
 
-        for serial in self._serial_tabs:
+        for serial in self._serial_consoles:
             if (serial.terminal and
                 serial.terminal.get_property("has-focus")):
                 return True
@@ -282,8 +356,9 @@ class vmmConsolePages(vmmGObjectUI):
     ###########################
 
     def _scroll_size_allocate(self, src_ignore, req):
-        if (not self._viewer or
-            not self._viewer.console_get_desktop_resolution()):
+        if not self._viewer:
+            return
+        if not self._viewer.console_get_desktop_resolution():
             return
 
         scroll = self.widget("console-gfx-scroll")
@@ -296,27 +371,24 @@ class vmmConsolePages(vmmGObjectUI):
 
         # pylint: disable=unpacking-non-sequence
         desktop_w, desktop_h = self._viewer.console_get_desktop_resolution()
-        if desktop_h == 0:
-            return
         desktop_ratio = float(desktop_w) / float(desktop_h)
 
-        if is_scale or self._force_resize:
+        if is_scale:
             # Make sure we never show scrollbars when scaling
             scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.NEVER)
         else:
             scroll.set_policy(Gtk.PolicyType.AUTOMATIC,
                               Gtk.PolicyType.AUTOMATIC)
 
-        if not self._force_resize and is_resizeguest:
+        if is_resizeguest:
             # With resize guest, we don't want to maintain aspect ratio,
             # since the guest can resize to arbitrary resolutions.
             self._viewer.console_set_size_request(req.width, req.height)
             return
 
-        if not is_scale or self._force_resize:
+        if not is_scale:
             # Scaling disabled is easy, just force the VNC widget size. Since
             # we are inside a scrollwindow, it shouldn't cause issues.
-            self._force_resize = False
             self._viewer.console_set_size_request(desktop_w, desktop_h)
             return
 
@@ -383,10 +455,14 @@ class vmmConsolePages(vmmGObjectUI):
         if not self._viewer.console_get_desktop_resolution():
             return
 
+        top_w, top_h = self.topwin.get_size()
+        viewer_alloc = self.widget("console-gfx-scroll").get_allocation()
+        desktop_w, desktop_h = self._viewer.console_get_desktop_resolution()
+
         self.topwin.unmaximize()
-        self.topwin.resize(1, 1)
-        self._force_resize = True
-        self.widget("console-gfx-scroll").queue_resize()
+        self.topwin.resize(
+            desktop_w + (top_w - viewer_alloc.width),
+            desktop_h + (top_h - viewer_alloc.height))
 
 
     ################
@@ -428,14 +504,14 @@ class vmmConsolePages(vmmGObjectUI):
         fs = self.widget("control-fullscreen").get_active()
         scale_type = self.vm.get_console_scaling()
 
-        if (scale_type == self.config.CONSOLE_SCALE_NEVER
-            and curscale is True):
+        if (scale_type == self.config.CONSOLE_SCALE_NEVER and
+            curscale is True):
             self._viewer.console_set_scaling(False)
-        elif (scale_type == self.config.CONSOLE_SCALE_ALWAYS
-              and curscale is False):
+        elif (scale_type == self.config.CONSOLE_SCALE_ALWAYS and
+              curscale is False):
             self._viewer.console_set_scaling(True)
-        elif (scale_type == self.config.CONSOLE_SCALE_FULLSCREEN
-              and curscale != fs):
+        elif (scale_type == self.config.CONSOLE_SCALE_FULLSCREEN and
+              curscale != fs):
             self._viewer.console_set_scaling(fs)
 
         # Refresh viewer size
@@ -450,9 +526,9 @@ class vmmConsolePages(vmmGObjectUI):
         cpage = self.widget("console-pages").get_current_page()
         dpage = self.widget("details-pages").get_current_page()
 
-        allow_fullscreen = (dpage == DETAILS_PAGE_CONSOLE and
-                            cpage == self.CONSOLE_PAGE_VIEWER and
-                            self._viewer_is_connected)
+        allow_fullscreen = bool(dpage == DETAILS_PAGE_CONSOLE and
+            cpage == _CONSOLE_PAGE_VIEWER and
+            self._viewer and self._viewer.console_is_open())
 
         self.widget("control-fullscreen").set_sensitive(allow_fullscreen)
         self.widget("details-menu-view-fullscreen").set_sensitive(
@@ -466,13 +542,11 @@ class vmmConsolePages(vmmGObjectUI):
 
         if do_fullscreen:
             self.topwin.fullscreen()
-            self._fs_toolbar.show()
-            self._fs_drawer.set_active(True)
+            self._timed_revealer.force_reveal(True)
             self.widget("toolbar-box").hide()
             self.widget("details-menubar").hide()
         else:
-            self._fs_toolbar.hide()
-            self._fs_drawer.set_active(False)
+            self._timed_revealer.force_reveal(False)
             self.topwin.unfullscreen()
 
             if self.widget("details-menu-view-toolbar").get_active():
@@ -488,9 +562,9 @@ class vmmConsolePages(vmmGObjectUI):
 
     def _show_vm_status_unavailable(self):
         if self.vm.is_crashed():
-            self._activate_unavailable_page(_("Guest has crashed"))
+            self._activate_unavailable_page(_("Guest has crashed."))
         else:
-            self._activate_unavailable_page(_("Guest not running"))
+            self._activate_unavailable_page(_("Guest is not running."))
 
     def _close_viewer(self):
         if self._viewer is None:
@@ -501,30 +575,27 @@ class vmmConsolePages(vmmGObjectUI):
         self._viewer.cleanup()
         self._viewer = None
 
-        self._viewer_is_connected = False
-        self._refresh_can_fullscreen()
         self._leave_fullscreen()
 
-        for serial in self._serial_tabs:
+        for serial in self._serial_consoles:
             serial.close()
 
     def _update_vm_widget_states(self):
-        runable = self.vm.is_runable()
-        paused = self.vm.is_paused()
-        pages = self.widget("console-pages")
-        page = pages.get_current_page()
+        page = self.widget("console-pages").get_current_page()
 
-        self._send_key_button.set_sensitive(not (runable or paused))
-
-        if runable:
+        if self.vm.is_runable():
             self._show_vm_status_unavailable()
 
-        elif page in [self.CONSOLE_PAGE_UNAVAILABLE, self.CONSOLE_PAGE_VIEWER]:
+        elif (page == _CONSOLE_PAGE_UNAVAILABLE or
+              page == _CONSOLE_PAGE_VIEWER):
             if self._viewer and self._viewer.console_is_open():
                 self._activate_viewer_page()
             else:
                 self._init_viewer()
 
+        # Update other state
+        self._refresh_widget_states()
+
 
     ###################
     # Page Navigation #
@@ -537,15 +608,12 @@ class vmmConsolePages(vmmGObjectUI):
         """
         self._close_viewer()
         self.widget("console-pages").set_current_page(
-            self.CONSOLE_PAGE_UNAVAILABLE)
-        self.widget("details-menu-vm-screenshot").set_sensitive(False)
-        self.widget("details-menu-usb-redirection").set_sensitive(False)
-        self.widget("console-unavailable").set_label("<b>" + msg + "</b>")
+            _CONSOLE_PAGE_UNAVAILABLE)
+        if msg:
+            self.widget("console-unavailable").set_label("<b>" + msg + "</b>")
 
     def _activate_auth_page(self, withPassword, withUsername):
         (pw, username) = self.config.get_console_password(self.vm)
-        self.widget("details-menu-vm-screenshot").set_sensitive(False)
-        self.widget("details-menu-usb-redirection").set_sensitive(False)
 
         self.widget("console-auth-password").set_visible(withPassword)
         self.widget("label-auth-password").set_visible(withPassword)
@@ -560,10 +628,10 @@ class vmmConsolePages(vmmGObjectUI):
                 bool(self.config.has_keyring()))
         if self.config.has_keyring():
             self.widget("console-auth-remember").set_active(
-                bool(pw and username))
+                    bool(withPassword and pw) or (withUsername and username))
 
         self.widget("console-pages").set_current_page(
-            self.CONSOLE_PAGE_AUTHENTICATE)
+            _CONSOLE_PAGE_AUTHENTICATE)
 
         if withUsername:
             self.widget("console-auth-username").grab_focus()
@@ -571,26 +639,39 @@ class vmmConsolePages(vmmGObjectUI):
             self.widget("console-auth-password").grab_focus()
 
     def _activate_viewer_page(self):
-        self.widget("console-pages").set_current_page(self.CONSOLE_PAGE_VIEWER)
-        self.widget("details-menu-vm-screenshot").set_sensitive(True)
+        self.widget("console-pages").set_current_page(_CONSOLE_PAGE_VIEWER)
         if self._viewer:
             self._viewer.console_grab_focus()
 
-        if (self._viewer.console_has_usb_redirection() and
-            self.vm.has_spicevmc_type_redirdev()):
-            self.widget("details-menu-usb-redirection").set_sensitive(True)
-            return
+    def _page_changed(self, src, origpage, newpage):
+        ignore = src
+        ignore = origpage
 
-    def _page_changed(self, ignore1=None, ignore2=None, newpage=None):
+        # Hide the contents of all other pages, so they don't screw
+        # up window sizing
+        for i in range(self.widget("console-pages").get_n_pages()):
+            self.widget("console-pages").get_nth_page(i).set_visible(
+                i == newpage)
+
+        self.idle_add(self._refresh_widget_states)
+
+    def _refresh_widget_states(self):
         pagenum = self.widget("console-pages").get_current_page()
+        paused = self.vm.is_paused()
+        is_viewer = bool(pagenum == _CONSOLE_PAGE_VIEWER and
+            self._viewer and self._viewer.console_is_open())
 
-        if newpage is not None:
-            for i in range(self.widget("console-pages").get_n_pages()):
-                w = self.widget("console-pages").get_nth_page(i)
-                w.set_visible(i == newpage)
+        self.widget("details-menu-vm-screenshot").set_sensitive(is_viewer)
+        self.widget("details-menu-usb-redirection").set_sensitive(
+            bool(is_viewer and self._viewer and
+            self._viewer.console_has_usb_redirection() and
+            self.vm.has_spicevmc_type_redirdev()))
+
+        can_sendkey = (is_viewer and not paused)
+        self._send_key_button.set_sensitive(can_sendkey)
+        for c in self._keycombo_menu.get_children():
+            c.set_sensitive(can_sendkey)
 
-        if pagenum < self.CONSOLE_PAGE_OFFSET:
-            self._last_gfx_page = pagenum
         self._refresh_can_fullscreen()
 
 
@@ -609,7 +690,7 @@ class vmmConsolePages(vmmGObjectUI):
             gdev = gdevs and gdevs[0] or None
             if gdev:
                 ginfo = ConnectionInfo(self.vm.conn, gdev)
-        except Exception, e:
+        except Exception as e:
             # We can fail here if VM is destroyed: xen is a bit racy
             # and can't handle domain lookups that soon after
             logging.exception("Getting graphics console failed: %s", str(e))
@@ -631,19 +712,6 @@ class vmmConsolePages(vmmGObjectUI):
             self._activate_unavailable_page(msg)
             return
 
-        if ginfo.is_bad_localhost():
-            self._activate_unavailable_page(
-                _("Guest is on a remote host with transport '%s'\n"
-                  "but is only configured to listen locally.\n"
-                  "Connect using 'ssh' transport or change the\n"
-                  "guest's listen address." % ginfo.transport))
-            return
-
-        if not ginfo.console_active():
-            self._activate_unavailable_page(
-                _("Graphical console is not yet active for guest"))
-            return
-
         self._activate_unavailable_page(
             _("Connecting to graphical console for guest"))
 
@@ -652,15 +720,20 @@ class vmmConsolePages(vmmGObjectUI):
             if ginfo.gtype == "vnc":
                 viewer_class = VNCViewer
             elif ginfo.gtype == "spice":
-                viewer_class = SpiceViewer
+                if have_spice_gtk:
+                    viewer_class = SpiceViewer
+                else:
+                    raise RuntimeError("Error opening Spice console, "
+                                       "SpiceClientGtk missing")
 
-            self._viewer = viewer_class(ginfo)
+
+            self._viewer = viewer_class(self.vm, ginfo)
             self._connect_viewer_signals()
 
             self._refresh_enable_accel()
 
             self._viewer.console_open()
-        except Exception, e:
+        except Exception as e:
             logging.exception("Error connection to graphical console")
             self._activate_unavailable_page(
                     _("Error connecting to graphical console") + ":\n%s" % e)
@@ -677,6 +750,8 @@ class vmmConsolePages(vmmGObjectUI):
         if self.widget("console-auth-remember").get_active():
             self.config.set_console_password(self.vm, passwd.get_text(),
                                              username.get_text())
+        else:
+            self.config.del_console_password(self.vm)
 
 
     ##########################
@@ -712,7 +787,6 @@ class vmmConsolePages(vmmGObjectUI):
             self._enable_modifiers()
 
     def _viewer_auth_rejected(self, ignore, errmsg):
-        self._close_viewer()
         self._activate_unavailable_page(errmsg)
 
     def _viewer_auth_error(self, ignore, errmsg, viewer_will_disconnect):
@@ -723,7 +797,6 @@ class vmmConsolePages(vmmGObjectUI):
             # GtkVNC will disconnect after an auth error, so lets do it for
             # them and re-init the viewer (which will be triggered by
             # update_vm_widget_states if needed)
-            self._close_viewer()
             self._activate_unavailable_page(errmsg)
 
         self._update_vm_widget_states()
@@ -738,36 +811,33 @@ class vmmConsolePages(vmmGObjectUI):
         self.err.show_err(_("USB redirection error"),
             text2=str(errstr), modal=True)
 
-    def _viewer_disconnected(self, ignore):
-        errout = ""
-        if self._viewer:
-            errout = self._viewer.console_get_err_output()
+    def _viewer_disconnected_set_page(self, errdetails, ssherr):
+        if self.vm.is_runable():
+            # Exit was probably for legitimate reasons
+            self._show_vm_status_unavailable()
+            return
 
-        self.widget("console-pages").set_current_page(
-            self.CONSOLE_PAGE_UNAVAILABLE)
-        self._close_viewer()
+        msg = _("Viewer was disconnected.")
+        if errdetails:
+            msg += "\n" + errdetails
+        if ssherr:
+            logging.debug("SSH tunnel error output: %s", ssherr)
+            msg += "\n\n"
+            msg += _("SSH tunnel error output: %s") % ssherr
+
+        self._activate_unavailable_page(msg)
+
+    def _viewer_disconnected(self, ignore, errdetails, ssherr):
+        self._activate_unavailable_page(_("Viewer disconnected."))
         logging.debug("Viewer disconnected")
 
         # Make sure modifiers are set correctly
         self._viewer_focus_changed()
 
-        if self.vm.is_runable():
-            # Exit was probably for legitimate reasons
-            self._show_vm_status_unavailable()
-        else:
-            error = _("Error: viewer connection to hypervisor host got "
-                "refused or disconnected!")
-            if errout:
-                logging.debug("Error output from closed console: %s", errout)
-                error += "\n\nError: %s" % errout
-            self._activate_unavailable_page(error)
-
+        self._viewer_disconnected_set_page(errdetails, ssherr)
         self._refresh_resizeguest_from_settings()
 
     def _viewer_connected(self, ignore):
-        self._viewer_is_connected = True
-        self._refresh_can_fullscreen()
-
         logging.debug("Viewer connected")
         self._activate_viewer_page()
 
@@ -796,9 +866,15 @@ class vmmConsolePages(vmmGObjectUI):
     ###########################
 
     def _activate_default_console_page(self):
+        """
+        Find the default graphical or serial console for the VM
+        """
         if self.vm.get_graphics_devices() or not self.vm.get_serial_devs():
             return
 
+        # We iterate through the 'console' menu and activate the first
+        # valid entry... it's the easiest thing to do to hit all the right
+        # code paths.
         self._populate_serial_menu()
         menu = self.widget("details-menu-view-serial-list").get_submenu()
         for child in menu.get_children():
@@ -808,34 +884,17 @@ class vmmConsolePages(vmmGObjectUI):
                 child.toggled()
                 break
 
-    def _selected_serial_dev(self):
-        current_page = self.widget("console-pages").get_current_page()
-        if not current_page >= self.CONSOLE_PAGE_OFFSET:
-            return
-
-        serial_idx = current_page - self.CONSOLE_PAGE_OFFSET
-        if len(self._serial_tabs) < serial_idx:
-            return
-
-        return self._serial_tabs[serial_idx]
-
-    def _make_serial_menu_label(self, dev):
-        if dev.virtual_device_type == "console":
-            return "Text Console %d" % (dev.vmmindex + 1)
-        return "Serial %d" % (dev.vmmindex + 1)
-
     def _console_menu_toggled(self, src, dev):
-        ignore = src
         self.widget("details-pages").set_current_page(DETAILS_PAGE_CONSOLE)
 
         if dev.virtual_device_type == "graphics":
-            self.widget("console-pages").set_current_page(self._last_gfx_page)
+            self.widget("console-pages").set_current_page(_CONSOLE_PAGE_VIEWER)
             return
 
         target_port = dev.vmmindex
-        name = self._make_serial_menu_label(dev)
         serial = None
-        for s in self._serial_tabs:
+        name = src.get_label()
+        for s in self._serial_consoles:
             if s.name == name:
                 serial = s
                 break
@@ -848,49 +907,51 @@ class vmmConsolePages(vmmGObjectUI):
                                     self._viewer_focus_changed)
 
             title = Gtk.Label(label=name)
-            self.widget("console-pages").append_page(serial.box, title)
-            self._serial_tabs.append(serial)
+            self.widget("serial-pages").append_page(serial.box, title)
+            self._serial_consoles.append(serial)
 
         serial.open_console()
-        page_idx = self._serial_tabs.index(serial) + self.CONSOLE_PAGE_OFFSET
-        self.widget("console-pages").set_current_page(page_idx)
+        page_idx = self._serial_consoles.index(serial)
+        self.widget("console-pages").set_current_page(_CONSOLE_PAGE_SERIAL)
+        self.widget("serial-pages").set_current_page(page_idx)
 
     def _build_serial_menu_items(self, menu_item_cb):
-        showing_serial_dev = self._selected_serial_dev()
         devs = self.vm.get_serial_devs()
-
         if len(devs) == 0:
             menu_item_cb(_("No text console available"),
                          radio=False, sensitive=False)
             return
 
+        active_label = None
+        if (self.widget("console-pages").get_current_page() ==
+                _CONSOLE_PAGE_SERIAL):
+            serial_page = self.widget("serial-pages").get_current_page()
+            if len(self._serial_consoles) > serial_page:
+                active_label = self._serial_consoles[serial_page].name
+
         for dev in devs:
-            label = self._make_serial_menu_label(dev)
+            if dev.virtual_device_type == "console":
+                label = _("Text Console %d") % (dev.vmmindex + 1)
+            else:
+                label = _("Serial %d") % (dev.vmmindex + 1)
+
             tooltip = vmmSerialConsole.can_connect(self.vm, dev)
             sensitive = not bool(tooltip)
 
-            active = (sensitive and
-                showing_serial_dev and
-                showing_serial_dev.name == label)
-
+            active = (sensitive and label == active_label)
             menu_item_cb(label, sensitive=sensitive, active=active,
                 tooltip=tooltip, cb=self._console_menu_toggled, cbdata=dev)
 
     def _build_graphical_menu_items(self, menu_item_cb):
-        showing_graphics = (
-            self.widget("console-pages").get_current_page() ==
-            self.CONSOLE_PAGE_VIEWER)
-
-        # Populate graphical devices
         devs = self.vm.get_graphics_devices()
         if len(devs) == 0:
             menu_item_cb(_("No graphical console available"),
                          radio=False, sensitive=False)
             return
 
-        # Only one graphical device supported for now
+        active = (self.widget("console-pages").get_current_page() !=
+                _CONSOLE_PAGE_SERIAL)
         for idx, dev in enumerate(devs):
-
             label = (_("Graphical Console") + " " +
                      dev.pretty_type_simple(dev.type))
 
@@ -902,7 +963,7 @@ class vmmConsolePages(vmmGObjectUI):
                 tooltip = _("virt-manager does not support more "
                             "that one graphical console")
 
-            menu_item_cb(label, active=showing_graphics,
+            menu_item_cb(label, active=active,
                 sensitive=sensitive, tooltip=tooltip,
                 cb=self._console_menu_toggled, cbdata=dev)
 
@@ -952,7 +1013,7 @@ class vmmConsolePages(vmmGObjectUI):
         return self._viewer.console_get_pixbuf()
 
     def details_close_viewer(self):
-        return self._close_viewer()
+        return self._activate_unavailable_page(_("Viewer disconnected."))
 
     def details_activate_default_console_page(self):
         return self._activate_default_console_page()
@@ -960,15 +1021,10 @@ class vmmConsolePages(vmmGObjectUI):
     def details_update_widget_states(self):
         return self._update_vm_widget_states()
 
-    def details_build_keycombo_menu(self, *args, **kwargs):
-        return self._build_keycombo_menu(*args, **kwargs)
-
     def details_refresh_can_fullscreen(self):
         return self._refresh_can_fullscreen()
     def details_resizeguest_ui_changed_cb(self, *args, **kwargs):
         return self._resizeguest_ui_changed_cb(*args, **kwargs)
-    def details_send_key(self, *args, **kwargs):
-        return self._do_send_key(*args, **kwargs)
 
     def details_page_changed(self, *args, **kwargs):
         return self._page_changed(*args, **kwargs)
