@@ -80,8 +80,8 @@ class vmmHost(vmmGObjectUI):
         self.init_interface_state()
 
         self.builder.connect_signals({
-            "on_menu_file_view_manager_activate" : self.view_manager,
-            "on_menu_file_quit_activate" : self.exit_app,
+            "on_menu_file_view_manager_activate": self.view_manager,
+            "on_menu_file_quit_activate": self.exit_app,
             "on_menu_file_close_activate": self.close,
             "on_vmm_host_delete_event": self.close,
             "on_host_page_switch": self.page_changed,
@@ -98,12 +98,12 @@ class vmmHost(vmmGObjectUI):
             "on_net_name_changed": (lambda *x:
                 self.enable_net_apply(x, EDIT_NET_NAME)),
 
-            "on_interface_add_clicked" : self.add_interface,
-            "on_interface_start_clicked" : self.start_interface,
-            "on_interface_stop_clicked" : self.stop_interface,
-            "on_interface_delete_clicked" : self.delete_interface,
+            "on_interface_add_clicked": self.add_interface,
+            "on_interface_start_clicked": self.start_interface,
+            "on_interface_stop_clicked": self.stop_interface,
+            "on_interface_delete_clicked": self.delete_interface,
             "on_interface_startmode_changed": self.interface_startmode_changed,
-            "on_interface_apply_clicked" : (lambda *x: self.interface_apply()),
+            "on_interface_apply_clicked": (lambda *x: self.interface_apply()),
             "on_interface_list_changed": self.interface_selected,
 
             "on_overview_name_changed": self._overview_name_changed,
@@ -407,7 +407,7 @@ class vmmHost(vmmGObjectUI):
             if self.addnet is None:
                 self.addnet = vmmCreateNetwork(self.conn)
             self.addnet.show(self.topwin)
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Error launching network wizard: %s") % str(e))
 
     def net_apply(self):
@@ -450,7 +450,7 @@ class vmmHost(vmmGObjectUI):
                         dialog_type=Gtk.MessageType.INFO)
 
 
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Error changing network settings: %s") % str(e))
             return
         finally:
@@ -481,10 +481,6 @@ class vmmHost(vmmGObjectUI):
             self.active_edits.append(edittype)
 
     def net_autostart_changed(self, src_ignore):
-        auto = self.widget("net-autostart").get_active()
-        self.widget("net-autostart").set_label(auto and
-                                               _("On Boot") or
-                                               _("Never"))
         self.enable_net_apply(EDIT_NET_AUTOSTART)
 
     def current_network(self):
@@ -522,7 +518,7 @@ class vmmHost(vmmGObjectUI):
         try:
             net = self.conn.get_net(connkey)
             self.populate_net_state(net)
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
             self.set_net_error_page(_("Error selecting network: %s") % e)
         finally:
@@ -640,9 +636,8 @@ class vmmHost(vmmGObjectUI):
         self.widget("net-delete").set_sensitive(not active)
 
         autostart = net.get_autostart()
-        autolabel = autostart and _("On Boot") or _("Never")
         self.widget("net-autostart").set_active(autostart)
-        self.widget("net-autostart").set_label(autolabel)
+        self.widget("net-autostart").set_label(_("On Boot"))
 
         self._populate_net_ipv4_state(net)
         self._populate_net_ipv6_state(net)
@@ -659,7 +654,7 @@ class vmmHost(vmmGObjectUI):
         self.widget("net-start").set_sensitive(False)
         self.widget("net-stop").set_sensitive(False)
         self.widget("net-delete").set_sensitive(False)
-        self.widget("net-autostart").set_label(_("Never"))
+        self.widget("net-autostart").set_label(_("On Boot"))
         self.widget("net-autostart").set_active(False)
         self.widget("net-ipv4-network").set_text("")
         self.widget("net-ipv4-dhcp-range").set_text("")
@@ -691,7 +686,7 @@ class vmmHost(vmmGObjectUI):
             for net in self.conn.list_nets():
                 try:
                     net.disconnect_by_func(self.refresh_network)
-                except:
+                except Exception:
                     pass
                 net.connect("state-changed", self.refresh_network)
                 model.append([net.get_connkey(), net.get_name(), "network-idle",
@@ -759,7 +754,7 @@ class vmmHost(vmmGObjectUI):
             if self.addinterface is None:
                 self.addinterface = vmmCreateInterface(self.conn)
             self.addinterface.show(self.topwin)
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Error launching interface wizard: %s") %
                               str(e))
 
@@ -786,7 +781,7 @@ class vmmHost(vmmGObjectUI):
                       interface.get_name())
         try:
             interface.set_startmode(newmode)
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Error setting interface startmode: %s") %
                               str(e))
             return
@@ -812,7 +807,7 @@ class vmmHost(vmmGObjectUI):
 
         try:
             self.populate_interface_state(connkey)
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
             self.set_interface_error_page(_("Error selecting interface: %s") %
                                           e)
@@ -864,7 +859,7 @@ class vmmHost(vmmGObjectUI):
         used_by = None
         try:
             used_by = vmmCreateInterface.iface_in_use_by(self.conn, name)
-        except Exception, e:
+        except Exception as e:
             logging.debug("Error looking up iface usage: %s", e)
         self.widget("interface-inuseby").set_text(used_by or "-")
 
@@ -941,7 +936,7 @@ class vmmHost(vmmGObjectUI):
             for iface in self.conn.list_interfaces():
                 try:
                     iface.disconnect_by_func(self.refresh_interface)
-                except:
+                except Exception:
                     pass
                 iface.connect("state-changed", self.refresh_interface)
                 model.append([iface.get_connkey(), iface.get_name(),
