@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2013 Red Hat, Inc.
+# Copyright (C) 2013-2014 Red Hat, Inc.
 # Copyright (C) 2013 Cole Robinson <crobinso@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -24,19 +24,17 @@ import logging
 import os
 import StringIO
 
-# pylint: disable=E0611
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Gtk
 from gi.repository import Pango
-# pylint: enable=E0611
 
 from virtinst import DomainSnapshot
 from virtinst import util
 
-from virtManager import uiutil
-from virtManager.baseclass import vmmGObjectUI
-from virtManager.asyncjob import vmmAsyncJob
+from . import uiutil
+from .baseclass import vmmGObjectUI
+from .asyncjob import vmmAsyncJob
 
 
 mimemap = {
@@ -64,6 +62,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         self.vm = vm
 
         self._initial_populate = False
+        self._unapplied_changes = False
 
         self._snapmenu = None
         self._init_ui()
@@ -80,19 +79,23 @@ class vmmSnapshotPage(vmmGObjectUI):
             "on_snapshot_apply_clicked": self._on_apply_clicked,
             "on_snapshot_list_changed": self._snapshot_selected,
             "on_snapshot_list_button_press_event": self._popup_snapshot_menu,
+            "on_snapshot_refresh_clicked": self._refresh_snapshots,
+            "on_snapshot_list_row_activated": self._on_start_clicked,
 
             # 'Create' dialog
             "on_snapshot_new_delete_event": self._snapshot_new_close,
             "on_snapshot_new_ok_clicked": self._on_new_ok_clicked,
-            "on_snapshot_new_cancel_clicked" : self._snapshot_new_close,
-            "on_snapshot_new_name_changed" : self._snapshot_new_name_changed,
+            "on_snapshot_new_cancel_clicked": self._snapshot_new_close,
+            "on_snapshot_new_name_changed": self._snapshot_new_name_changed,
             "on_snapshot_new_name_activate": self._on_new_ok_clicked,
         })
 
         self.top_box = self.widget("snapshot-top-box")
         self.widget("snapshot-top-window").remove(self.top_box)
-        self.widget("snapshot-list").get_selection().emit("changed")
-
+        selection = self.widget("snapshot-list").get_selection()
+        selection.emit("changed")
+        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
+        selection.set_select_function(self._confirm_changes, None)
 
     ##############
     # Init stuff #
@@ -105,6 +108,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         self._snapshot_new = None
 
     def _init_ui(self):
+        # pylint: disable=redefined-variable-type
         blue = Gdk.color_parse("#0072A8")
         self.widget("header").modify_bg(Gtk.StateType.NORMAL, blue)
 
@@ -179,18 +183,21 @@ class vmmSnapshotPage(vmmGObjectUI):
     # Functional bits #
     ###################
 
-    def _get_selected_snapshot(self):
-        name = uiutil.get_list_selection(self.widget("snapshot-list"), 0)
-        if not name:
-            return None
+    def _get_selected_snapshots(self):
+        selection = self.widget("snapshot-list").get_selection()
+        def add_snap(treemodel, path, it, snaps):
+            ignore = path
+            try:
+                name = treemodel[it][0]
+                for snap in self.vm.list_snapshots():
+                    if name == snap.get_name():
+                        snaps.append(snap)
+            except Exception:
+                pass
 
-        try:
-            for snap in self.vm.list_snapshots():
-                if name == snap.get_name():
-                    return snap
-        except:
-            pass
-        return None
+        snaps = []
+        selection.selected_foreach(add_snap, snaps)
+        return snaps
 
     def _refresh_snapshots(self, select_name=None):
         self.vm.refresh_snapshots()
@@ -206,13 +213,16 @@ class vmmSnapshotPage(vmmGObjectUI):
         self.widget("snapshot-error-label").set_text(msg)
 
     def _populate_snapshot_list(self, select_name=None):
-        cursnap = self._get_selected_snapshot()
+        cursnaps = []
+        for i in self._get_selected_snapshots():
+            cursnaps.append(i.get_name())
+
         model = self.widget("snapshot-list").get_model()
         model.clear()
 
         try:
             snapshots = self.vm.list_snapshots()
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
             self._set_error_page(_("Error refreshing snapshot list: %s") %
                                 str(e))
@@ -222,11 +232,8 @@ class vmmSnapshotPage(vmmGObjectUI):
         has_internal = False
         for snap in snapshots:
             desc = snap.get_xmlobj().description
-            if not uiutil.can_set_row_none:
-                desc = desc or ""
-
             name = snap.get_name()
-            state = util.xml_escape(snap.run_status())
+            state = snap.run_status()
             if snap.is_external():
                 has_external = True
                 sortname = "3%s" % name
@@ -237,15 +244,27 @@ class vmmSnapshotPage(vmmGObjectUI):
                 sortname = "1%s" % name
 
             label = "%s\n<span size='small'>%s: %s%s</span>" % (
-                (name, _("VM State"), state, external))
+                (util.xml_escape(name), _("VM State"),
+                 util.xml_escape(state), external))
             model.append([name, label, desc, snap.run_status_icon_name(),
                           sortname, snap.is_current()])
 
         if has_internal and has_external:
             model.append([None, None, None, None, "2", False])
 
-        select_name = select_name or (cursnap and cursnap.get_name() or None)
-        uiutil.set_row_selection(self.widget("snapshot-list"), select_name)
+
+        def check_selection(treemodel, path, it, snaps):
+            if select_name:
+                if treemodel[it][0] == select_name:
+                    selection.select_path(path)
+            elif treemodel[it][0] in snaps:
+                selection.select_path(path)
+
+        selection = self.widget("snapshot-list").get_selection()
+        model = self.widget("snapshot-list").get_model()
+        selection.unselect_all()
+        model.foreach(check_selection, cursnaps)
+
         self._initial_populate = True
 
     def _make_screenshot_pixbuf(self, mime, sdata):
@@ -285,7 +304,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         mime = _mime_to_ext(os.path.splitext(filename)[1][1:], reverse=True)
         if not mime:
             return
-        return self._make_screenshot_pixbuf(mime, file(filename, "rb").read())
+        return self._make_screenshot_pixbuf(mime, open(filename, "rb").read())
 
     def _set_snapshot_state(self, snap=None):
         self.widget("snapshot-notebook").set_current_page(0)
@@ -341,7 +360,27 @@ class vmmSnapshotPage(vmmGObjectUI):
         self.widget("snapshot-delete").set_sensitive(bool(snap))
         self.widget("snapshot-start").set_sensitive(bool(snap))
         self.widget("snapshot-apply").set_sensitive(False)
+        self._unapplied_changes = False
 
+    def _confirm_changes(self, sel, model, path, path_selected, user_data):
+        ignore1 = sel
+        ignore2 = path
+        ignore3 = model
+        ignore4 = user_data
+
+        if not self._unapplied_changes or not path_selected:
+            return True
+
+        if self.err.chkbox_helper(
+                self.config.get_confirm_unapplied,
+                self.config.set_confirm_unapplied,
+                text1=(_("There are unapplied changes. "
+                         "Would you like to apply them now?")),
+                chktext=_("Don't warn me again."),
+                default=False):
+            self._apply()
+
+        return True
 
     ##################
     # 'New' handling #
@@ -367,7 +406,7 @@ class vmmSnapshotPage(vmmGObjectUI):
             try:
                 if stream:
                     stream.finish()
-            except:
+            except Exception:
                 pass
 
     def _get_screenshot(self):
@@ -379,8 +418,13 @@ class vmmSnapshotPage(vmmGObjectUI):
             return
 
         try:
+            # Perform two screenshots, because qemu + qxl has a bug where
+            # screenshot generally only shows the data from the previous
+            # screenshot request:
+            # https://bugs.launchpad.net/qemu/+bug/1314293
+            self._take_screenshot()
             mime, sdata = self._take_screenshot()
-        except:
+        except Exception:
             logging.exception("Error taking screenshot")
             return
 
@@ -417,9 +461,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         self.widget("snapshot-new-ok").set_sensitive(bool(src.get_text()))
 
     def _new_finish_cb(self, error, details, newname):
-        self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(
-                Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.reset_finish_cursor()
 
         if error is not None:
             error = _("Error creating snapshot: %s") % error
@@ -430,7 +472,7 @@ class vmmSnapshotPage(vmmGObjectUI):
     def _validate_new_snapshot(self):
         name = self.widget("snapshot-new-name").get_text()
         desc = self.widget("snapshot-new-description"
-            ).get_buffer().get_property("text")
+                           ).get_buffer().get_property("text")
 
         try:
             newsnap = DomainSnapshot(self.vm.conn.get_backend())
@@ -439,8 +481,8 @@ class vmmSnapshotPage(vmmGObjectUI):
             newsnap.validate()
             newsnap.get_xml_config()
             return newsnap
-        except Exception, e:
-            return self.err.val_err(_("Error validating snapshot: %s" % e))
+        except Exception as e:
+            return self.err.val_err(_("Error validating snapshot: %s") % e)
 
     def _get_screenshot_data_for_save(self):
         snwidget = self.widget("snapshot-new-screenshot")
@@ -475,8 +517,8 @@ class vmmSnapshotPage(vmmGObjectUI):
 
             filename = basesn + "." + _mime_to_ext(mime)
             logging.debug("Writing screenshot to %s", filename)
-            file(filename, "wb").write(sndata)
-        except:
+            open(filename, "wb").write(sndata)
+        except Exception:
             logging.exception("Error saving screenshot")
 
     def _create_new_snapshot(self):
@@ -487,12 +529,9 @@ class vmmSnapshotPage(vmmGObjectUI):
         xml = snap.get_xml_config()
         name = snap.name
         mime, sndata = self._get_screenshot_data_for_save()
-
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(
-                Gdk.Cursor.new(Gdk.CursorType.WATCH))
-
         self._snapshot_new_close()
+
+        self.set_finish_cursor()
         progWin = vmmAsyncJob(
                     self._do_create_snapshot, [xml, name, mime, sndata],
                     self._new_finish_cb, [name],
@@ -501,6 +540,28 @@ class vmmSnapshotPage(vmmGObjectUI):
                     self.topwin)
         progWin.run()
 
+    def _apply(self):
+        snaps = self._get_selected_snapshots()
+        if not snaps or len(snaps) > 1:
+            return False
+
+        snap = snaps[0]
+        desc_widget = self.widget("snapshot-description")
+        desc = desc_widget.get_buffer().get_property("text") or ""
+
+        xmlobj = snap.get_xmlobj()
+        origxml = xmlobj.get_xml_config()
+        xmlobj.description = desc
+        newxml = xmlobj.get_xml_config()
+
+        self.vm.log_redefine_xml_diff(snap, origxml, newxml)
+        if newxml == origxml:
+            return True
+
+        self.vm.create_snapshot(newxml, redefine=True)
+        snap.ensure_latest_xml()
+        return True
+
 
     #############
     # Listeners #
@@ -519,26 +580,17 @@ class vmmSnapshotPage(vmmGObjectUI):
         return 1
 
     def _description_changed(self, ignore):
-        self.widget("snapshot-apply").set_sensitive(True)
-
-    def _on_apply_clicked(self, ignore):
-        snap = self._get_selected_snapshot()
-        if not snap:
-            return
-
+        snaps = self._get_selected_snapshots()
         desc_widget = self.widget("snapshot-description")
         desc = desc_widget.get_buffer().get_property("text") or ""
 
-        xmlobj = snap.get_xmlobj()
-        origxml = xmlobj.get_xml_config()
-        xmlobj.description = desc
-        newxml = xmlobj.get_xml_config()
+        if len(snaps) == 1 and snaps[0].get_xmlobj().description != desc:
+            self._unapplied_changes = True
 
-        self.vm.log_redefine_xml_diff(snap, origxml, newxml)
-        if newxml == origxml:
-            return
-        self.vm.create_snapshot(newxml, redefine=True)
-        snap.refresh_xml()
+        self.widget("snapshot-apply").set_sensitive(True)
+
+    def _on_apply_clicked(self, ignore):
+        self._apply()
         self._refresh_snapshots()
 
     def _on_new_ok_clicked(self, ignore):
@@ -551,12 +603,21 @@ class vmmSnapshotPage(vmmGObjectUI):
         self._snapshot_new.show()
         self.widget("snapshot-new-name").grab_focus()
 
-    def _on_start_clicked(self, ignore):
-        snap = self._get_selected_snapshot()
-        result = self.err.yes_no(_("Are you sure you want to run "
-                                   "snapshot '%s'? All disk changes since "
-                                   "the last snapshot was created will be "
-                                   "discarded.") % snap.get_name())
+    def _on_start_clicked(self, ignore, ignore2=None, ignore3=None):
+        snaps = self._get_selected_snapshots()
+        if not snaps or len(snaps) > 1:
+            return
+
+        snap = snaps[0]
+        msg = _("Are you sure you want to run snapshot '%s'? "
+            "All %s changes since the last snapshot was created will be "
+            "discarded.")
+        if self.vm.is_active():
+            msg = msg % (snap.get_name(), _("disk"))
+        else:
+            msg = msg % (snap.get_name(), _("disk and configuration"))
+
+        result = self.err.yes_no(msg)
         if not result:
             return
 
@@ -570,33 +631,39 @@ class vmmSnapshotPage(vmmGObjectUI):
                             finish_cb=self._refresh_snapshots)
 
     def _on_delete_clicked(self, ignore):
-        snap = self._get_selected_snapshot()
-        if not snap:
+        snaps = self._get_selected_snapshots()
+        if not snaps:
             return
 
         result = self.err.yes_no(_("Are you sure you want to permanently "
-                                   "delete the snapshot '%s'?") %
-                                   snap.get_name())
+                                   "delete the selected snapshots?"))
         if not result:
             return
 
-        logging.debug("Deleting snapshot '%s'", snap.get_name())
-        vmmAsyncJob.simple_async(snap.delete, [], self,
-                        _("Deleting snapshot"),
-                        _("Deleting snapshot '%s'") % snap.get_name(),
-                        _("Error deleting snapshot '%s'") % snap.get_name(),
-                        finish_cb=self._refresh_snapshots)
+        for snap in snaps:
+            logging.debug("Deleting snapshot '%s'", snap.get_name())
+            vmmAsyncJob.simple_async(snap.delete, [], self,
+                            _("Deleting snapshot"),
+                            _("Deleting snapshot '%s'") % snap.get_name(),
+                            _("Error deleting snapshot '%s'") % snap.get_name(),
+                            finish_cb=self._refresh_snapshots)
 
 
     def _snapshot_selected(self, selection):
         ignore = selection
-        snap = self._get_selected_snapshot()
+        snap = self._get_selected_snapshots()
         if not snap:
             self._set_error_page(_("No snapshot selected."))
             return
+        if len(snap) > 1:
+            self._set_error_page(_("Multiple snapshots selected."))
+            self.widget("snapshot-start").set_sensitive(False)
+            self.widget("snapshot-apply").set_sensitive(False)
+            self.widget("snapshot-delete").set_sensitive(True)
+            return
 
         try:
-            self._set_snapshot_state(snap)
-        except Exception, e:
+            self._set_snapshot_state(snap[0])
+        except Exception as e:
             logging.exception(e)
             self._set_error_page(_("Error selecting snapshot: %s") % str(e))
