@@ -20,8 +20,8 @@
 # MA 02110-1301 USA.
 
 import logging
+import os
 
-import urlgrabber.progress as progress
 import libvirt
 
 from virtcli import CLIConfig
@@ -38,7 +38,9 @@ from .devicecontroller import VirtualController
 from .devicedisk import VirtualDisk
 from .devicegraphics import VirtualGraphics
 from .deviceinput import VirtualInputDevice
+from .devicepanic import VirtualPanicDevice
 from .deviceredirdev import VirtualRedirDevice
+from .devicerng import VirtualRNGDevice
 from .devicevideo import VirtualVideoDevice
 from .distroinstaller import DistroInstaller
 from .domainblkiotune import DomainBlkiotune
@@ -52,7 +54,9 @@ from .idmap import IdMap
 from .osxml import OSXML
 from .pm import PM
 from .seclabel import Seclabel
+from .sysinfo import SYSInfo
 from .xmlbuilder import XMLBuilder, XMLProperty, XMLChildProperty
+from .xmlnsqemu import XMLNSQemu
 
 
 class Guest(XMLBuilder):
@@ -82,7 +86,7 @@ class Guest(XMLBuilder):
 
             logging.info("Undefining guest '%s'", name)
             vm.undefine()
-        except libvirt.libvirtError, e:
+        except libvirt.libvirtError as e:
             raise RuntimeError(_("Could not remove old vm '%s': %s") %
                                (str(e)))
 
@@ -95,17 +99,18 @@ class Guest(XMLBuilder):
 
         try:
             conn.lookupByName(name)
-        except:
+        except Exception:
             return
         raise ValueError(_("Guest name '%s' is already in use.") % name)
 
 
     _XML_ROOT_NAME = "domain"
     _XML_PROP_ORDER = ["type", "name", "uuid", "title", "description",
-        "maxmemory", "memory", "blkiotune", "memtune", "memoryBacking",
-        "vcpus", "curvcpus", "numatune", "bootloader", "os", "idmap",
+        "hotplugmemorymax", "hotplugmemoryslots", "maxmemory", "memory", "blkiotune",
+        "memtune", "memoryBacking", "vcpus", "curvcpus", "vcpu_placement",
+        "cpuset", "numatune", "resource", "sysinfo", "bootloader", "os", "idmap",
         "features", "cpu", "clock", "on_poweroff", "on_reboot", "on_crash",
-        "resource", "pm", "emulator", "_devices", "seclabel"]
+        "pm", "emulator", "_devices", "seclabels"]
 
     def __init__(self, *args, **kwargs):
         XMLBuilder.__init__(self, *args, **kwargs)
@@ -121,6 +126,7 @@ class Guest(XMLBuilder):
         self.skip_default_sound = False
         self.skip_default_usbredir = False
         self.skip_default_graphics = False
+        self.skip_default_rng = False
         self.x86_cpu_default = self.cpu.SPECIAL_MODE_HOST_MODEL_ONLY
 
         self.__os_object = None
@@ -131,6 +137,9 @@ class Guest(XMLBuilder):
         # The libvirt virDomain object we 'Create'
         self.domain = None
 
+        # This is set via Capabilities.build_virtinst_guest
+        self.capsinfo = None
+
         self.installer = DistroInstaller(self.conn)
 
 
@@ -155,6 +164,8 @@ class Guest(XMLBuilder):
                          default_cb=lambda s: 1,
                          set_converter=_set_memory)
     maxmemory = XMLProperty("./memory", is_int=True)
+    hotplugmemorymax = XMLProperty("./maxMemory", is_int=True)
+    hotplugmemoryslots = XMLProperty("./maxMemory/@slots", is_int=True)
 
     def _set_vcpus(self, val):
         if val is None:
@@ -168,6 +179,7 @@ class Guest(XMLBuilder):
                         set_converter=_set_vcpus,
                         default_cb=lambda s: 1)
     curvcpus = XMLProperty("./vcpu/@current", is_int=True)
+    vcpu_placement = XMLProperty("./vcpu/@placement")
 
     def _validate_cpuset(self, val):
         DomainNumatune.validate_cpuset(self.conn, val)
@@ -182,6 +194,7 @@ class Guest(XMLBuilder):
                        validate_cb=lambda s, v: util.validate_uuid(v),
                        default_cb=_get_default_uuid)
 
+
     id = XMLProperty("./@id", is_int=True)
     type = XMLProperty("./@type", default_cb=lambda s: "xen")
     bootloader = XMLProperty("./bootloader")
@@ -189,15 +202,15 @@ class Guest(XMLBuilder):
     title = XMLProperty("./title")
     emulator = XMLProperty("./devices/emulator")
 
-    on_poweroff = XMLProperty("./on_poweroff",
-                              default_cb=lambda s: "destroy")
-    on_reboot = XMLProperty("./on_reboot", default_cb=lambda s: "restart")
-    on_crash = XMLProperty("./on_crash", default_cb=lambda s: "restart")
+    on_poweroff = XMLProperty("./on_poweroff")
+    on_reboot = XMLProperty("./on_reboot")
+    on_crash = XMLProperty("./on_crash")
+    on_lockfailure = XMLProperty("./on_lockfailure")
 
+    seclabels = XMLChildProperty(Seclabel)
     os = XMLChildProperty(OSXML, is_single=True)
     features = XMLChildProperty(DomainFeatures, is_single=True)
     clock = XMLChildProperty(Clock, is_single=True)
-    seclabel = XMLChildProperty(Seclabel, is_single=True)
     cpu = XMLChildProperty(CPU, is_single=True)
     numatune = XMLChildProperty(DomainNumatune, is_single=True)
     pm = XMLChildProperty(PM, is_single=True)
@@ -206,6 +219,9 @@ class Guest(XMLBuilder):
     memoryBacking = XMLChildProperty(DomainMemorybacking, is_single=True)
     idmap = XMLChildProperty(IdMap, is_single=True)
     resource = XMLChildProperty(DomainResource, is_single=True)
+    sysinfo = XMLChildProperty(SYSInfo, is_single=True)
+
+    xmlns_qemu = XMLChildProperty(XMLNSQemu, is_single=True)
 
 
     ###############################
@@ -247,7 +263,7 @@ class Guest(XMLBuilder):
 
         @param dev: VirtualDevice instance to attach to guest
         """
-        self._add_child(dev)
+        self.add_child(dev)
 
     def remove_device(self, dev):
         """
@@ -255,7 +271,7 @@ class Guest(XMLBuilder):
 
         @param dev: VirtualDevice instance
         """
-        self._remove_child(dev)
+        self.remove_child(dev)
 
     def get_devices(self, devtype):
         """
@@ -270,6 +286,7 @@ class Guest(XMLBuilder):
             if devtype == "all" or i.virtual_device_type == devtype:
                 newlist.append(i)
         return newlist
+
     _devices = XMLChildProperty(
         [VirtualDevice.virtual_device_classes[_n]
          for _n in VirtualDevice.virtual_device_types],
@@ -304,17 +321,16 @@ class Guest(XMLBuilder):
         # We do a shallow copy of the OS block here, so that we can
         # set the install time properties but not permanently overwrite
         # any config the user explicitly requested.
-        data = (self.os, self.on_crash, self.on_reboot)
+        data = (self.os, self.on_reboot)
         try:
             self._propstore["os"] = self.os.copy()
-        except:
+        except Exception:
             self._finish_get_xml(data)
             raise
         return data
 
     def _finish_get_xml(self, data):
         (self._propstore["os"],
-         self.on_crash,
          self.on_reboot) = data
 
     def _get_install_xml(self, *args, **kwargs):
@@ -324,33 +340,25 @@ class Guest(XMLBuilder):
         finally:
             self._finish_get_xml(data)
 
-    def _do_get_install_xml(self, install=True, disk_boot=False):
+    def _do_get_install_xml(self, install):
         """
         Return the full Guest xml configuration.
 
-        @param install: Whether we want the 'OS install' configuration or
-                        the 'post-install' configuration. (Some installs,
-                        like an import or livecd may not have an 'install'
-                        config.)
-        @type install: C{bool}
-        @param disk_boot: Whether we should boot off the harddisk, regardless
-                          of our position in the install process (this is
-                          used for 2 stage installs, where the second stage
-                          boots off the disk. You probably don't need to touch
-                          this.)
-        @type disk_boot: C{bool}
+        @install: Whether we want the 'OS install' configuration or
+            the 'post-install' configuration. The difference is mostly
+            whether the install media is attached and set as the boot
+            device. Some installs, like an import or livecd, do not have
+            an 'install' config.
         """
-        osblob_install = install and not disk_boot
-        if osblob_install and not self.installer.has_install_phase():
+        if install and not self.installer.has_install_phase():
             return None
 
-        self.installer.alter_bootconfig(self, osblob_install)
+        self.installer.alter_bootconfig(self, install)
         if not install:
             self._remove_cdrom_install_media()
 
         if install:
             self.on_reboot = "destroy"
-            self.on_crash = "destroy"
 
         self._set_osxml_defaults()
 
@@ -364,135 +372,75 @@ class Guest(XMLBuilder):
         return self.get_xml_config()
 
 
-    ##############
-    # Public API #
-    ##############
+    ###########################
+    # Private install helpers #
+    ###########################
 
-    def get_continue_inst(self):
-        """
-        Return True if this guest requires a call to 'continue_install',
-        which means the OS requires a 2 stage install (windows)
-        """
-        # If we are doing an 'import' or 'liveCD' install, there is
-        # no true install process, so continue install has no meaning
-        if not self.installer.has_install_phase():
-            return False
-
-        return self._os_object.is_windows()
-
-    def start_install(self, meter=None,
-                      dry=False, return_xml=False, noboot=False):
-        """
-        Begin the guest install (stage1).
-        @param return_xml: Don't create the guest, just return generated XML
-        """
-        if self.domain is not None:
-            raise RuntimeError(_("Domain has already been started!"))
-
-        is_initial = True
-        self.set_install_defaults()
-
-        self._prepare_install(meter, dry)
-        try:
-            # Create devices if required (disk images, etc.)
-            if not dry:
-                for dev in self.get_all_devices():
-                    dev.setup(meter)
-
-            start_xml, final_xml = self._build_xml(is_initial)
-            if return_xml:
-                return (start_xml, final_xml)
-            if dry:
-                return
-
-            # Remove existing VM if requested
-            self.check_vm_collision(self.conn, self.name,
-                                    do_remove=self.replace)
-
-            self.domain = self._create_guest(meter,
-                                             start_xml, final_xml, is_initial,
-                                             noboot)
-            # Set domain autostart flag if requested
-            self._flag_autostart()
-
-            return self.domain
-        finally:
-            self.installer.cleanup()
-
-    def continue_install(self, meter=None,
-                         dry=False, return_xml=False):
-        """
-        Continue with stage 2 of a guest install. Only required for
-        guests which have the 'continue' flag set (accessed via
-        get_continue_inst)
-        """
-        is_initial = False
-        start_xml, final_xml = self._build_xml(is_initial)
-        if return_xml:
-            return (start_xml, final_xml)
-        if dry:
-            return
-
-        return self._create_guest(meter,
-                                  start_xml, final_xml, is_initial, False)
-
-    def _build_meter(self, meter, is_initial):
-        if is_initial:
-            meter_label = _("Creating domain...")
-        else:
-            meter_label = _("Starting domain...")
-
-        if meter is None:
-            meter = progress.BaseMeter()
-        meter.start(size=None, text=meter_label)
-
-        return meter
-
-    def _build_xml(self, is_initial):
-        log_label = is_initial and "install" or "continue"
-        disk_boot = not is_initial
-
-        start_xml = self._get_install_xml(install=True, disk_boot=disk_boot)
+    def _build_xml(self):
+        install_xml = self._get_install_xml(install=True)
         final_xml = self._get_install_xml(install=False)
 
-        logging.debug("Generated %s XML: %s",
-                      log_label,
-                      (start_xml and ("\n" + start_xml) or "None required"))
+        logging.debug("Generated install XML: %s",
+            (install_xml and ("\n" + install_xml) or "None required"))
         logging.debug("Generated boot XML: \n%s", final_xml)
 
-        return start_xml, final_xml
+        return install_xml, final_xml
 
-    def _create_guest(self, meter,
-                      start_xml, final_xml, is_initial, noboot):
+    def _manual_transient_create(self, install_xml, final_xml, needs_boot):
+        """
+        For hypervisors (like vz) that don't implement createXML,
+        we need to define+start, and undefine on start failure
+        """
+        domain = self.conn.defineXML(install_xml or final_xml)
+        if not needs_boot:
+            return domain
+
+        # Handle undefining the VM if the initial startup fails
+        try:
+            domain.create()
+        except Exception:
+            import sys
+            exc_info = sys.exc_info()
+            try:
+                domain.undefine()
+            except Exception:
+                pass
+            raise exc_info[0], exc_info[1], exc_info[2]
+
+        if install_xml and install_xml != final_xml:
+            domain = self.conn.defineXML(final_xml)
+        return domain
+
+    def _create_guest(self, meter, install_xml, final_xml, doboot, transient):
         """
         Actually do the XML logging, guest defining/creating
 
-        @param is_initial: If running initial guest creation, else we
-                           are continuing the install
-        @param noboot: Don't boot guest if no install phase
+        @param doboot: Boot guest even if it has no install phase
         """
-        meter = self._build_meter(meter, is_initial)
-        doboot = not noboot or self.installer.has_install_phase()
+        meter_label = _("Creating domain...")
+        meter = util.ensure_meter(meter)
+        meter.start(size=None, text=meter_label)
+        needs_boot = doboot or self.installer.has_install_phase()
+
+        if self.type == "vz":
+            if transient:
+                raise RuntimeError(_("Domain type 'vz' doesn't support "
+                    "transient installs."))
+            domain = self._manual_transient_create(
+                    install_xml, final_xml, needs_boot)
 
-        if is_initial and doboot:
-            dom = self.conn.createLinux(start_xml or final_xml, 0)
         else:
-            dom = self.conn.defineXML(start_xml or final_xml)
-            if doboot:
-                dom.create()
+            if transient or needs_boot:
+                domain = self.conn.createXML(install_xml or final_xml, 0)
+            if not transient:
+                domain = self.conn.defineXML(final_xml)
 
-        self.domain = dom
-        meter.end(0)
-
-        self.domain = self.conn.defineXML(final_xml)
-        if is_initial:
-            try:
-                logging.debug("XML fetched from libvirt object:\n%s",
-                              dom.XMLDesc(0))
-            except Exception, e:
-                logging.debug("Error fetching XML from libvirt object: %s", e)
-
-        return self.domain
+        self.domain = domain
+        try:
+            logging.debug("XML fetched from libvirt object:\n%s",
+                          self.domain.XMLDesc(0))
+        except Exception as e:
+            logging.debug("Error fetching XML from libvirt object: %s", e)
 
 
     def _flag_autostart(self):
@@ -504,14 +452,88 @@ class Guest(XMLBuilder):
 
         try:
             self.domain.setAutostart(True)
-        except libvirt.libvirtError, e:
+        except libvirt.libvirtError as e:
             if util.is_error_nosupport(e):
-                logging.warn("Could not set autostart flag: libvirt "
+                logging.warning("Could not set autostart flag: libvirt "
                              "connection does not support autostart.")
             else:
                 raise e
 
 
+
+    ##############
+    # Public API #
+    ##############
+
+    def start_install(self, meter=None,
+                      dry=False, return_xml=False,
+                      doboot=True, transient=False):
+        """
+        Begin the guest install (stage1).
+        @param return_xml: Don't create the guest, just return generated XML
+        """
+        if self.domain is not None:
+            raise RuntimeError(_("Domain has already been started!"))
+
+        self.set_install_defaults()
+
+        self._prepare_install(meter, dry)
+        try:
+            # Create devices if required (disk images, etc.)
+            if not dry:
+                for dev in self.get_all_devices():
+                    dev.setup(meter)
+
+            install_xml, final_xml = self._build_xml()
+            if return_xml:
+                return (install_xml, final_xml)
+            if dry:
+                return
+
+            # Remove existing VM if requested
+            self.check_vm_collision(self.conn, self.name,
+                                    do_remove=self.replace)
+
+            self._create_guest(meter, install_xml, final_xml,
+                               doboot, transient)
+
+            # Set domain autostart flag if requested
+            self._flag_autostart()
+        finally:
+            self.installer.cleanup()
+
+    def get_created_disks(self):
+        return [d for d in self.get_devices("disk") if d.storage_was_created]
+
+    def cleanup_created_disks(self, meter):
+        """
+        Remove any disks we created as part of the install. Only ever
+        called by clients.
+        """
+        clean_disks = self.get_created_disks()
+        if not clean_disks:
+            return
+
+        for disk in clean_disks:
+            logging.debug("Removing created disk path=%s vol_object=%s",
+                disk.path, disk.get_vol_object())
+            name = os.path.basename(disk.path)
+
+            try:
+                meter.start(size=None, text=_("Removing disk '%s'") % name)
+
+                if disk.get_vol_object():
+                    disk.get_vol_object().delete()
+                else:
+                    os.unlink(disk.path)
+
+                meter.end(0)
+            except Exception as e:
+                logging.debug("Failed to remove disk '%s'",
+                    name, exc_info=True)
+                logging.error("Failed to remove disk '%s': %s", name, e)
+
+
     ###########################
     # XML convenience helpers #
     ###########################
@@ -540,6 +562,33 @@ class Guest(XMLBuilder):
         self.os.loader_type = "pflash"
         self.os.loader = path
 
+        self.check_uefi_secure()
+
+
+    def check_uefi_secure(self):
+        """
+        If the firmware name contains "secboot" it is probably build
+        with SMM feature required so we need to enable that feature,
+        otherwise the firmware may fail to load.  True secure boot is
+        currently supported only on x86 architecture and with q35 with
+        SMM feature enabled so change the machine to q35 as well.
+        To actually enforce the secure boot for the guest if Secure Boot
+        Mode is configured we need to enable loader secure feature.
+        """
+
+        if not self.os.is_x86():
+            return
+
+        if "secboot" not in self.os.loader:
+            return
+
+        if (not self.conn.check_support(self.conn.SUPPORT_DOMAIN_FEATURE_SMM) or
+            not self.conn.check_support(self.conn.SUPPORT_DOMAIN_LOADER_SECURE)):
+            return
+
+        self.features.smm = True
+        self.os.loader_secure = True
+        self.os.machine = "q35"
 
     ###################
     # Device defaults #
@@ -563,39 +612,52 @@ class Guest(XMLBuilder):
     def stable_defaults(self, *args, **kwargs):
         return self.conn.stable_defaults(self.emulator, *args, **kwargs)
 
+    def _usb_disabled(self):
+        controllers = [c for c in self.get_devices("controller") if
+            c.type == "usb"]
+        if not controllers:
+            return False
+        return all([c.model == "none" for c in controllers])
+
     def add_default_input_device(self):
         if self.os.is_container():
             return
-        if not self.os.is_x86():
-            return
         if self.get_devices("input"):
             return
-        self.add_device(VirtualInputDevice(self.conn))
+        if not self.get_devices("graphics"):
+            return
+        if self._usb_disabled():
+            return
 
-    def add_default_sound_device(self):
-        if not self.os.is_hvm():
-            return
-        if not self.os.is_x86():
-            return
-        self.add_device(VirtualAudio(self.conn))
+        usb_tablet = False
+        usb_keyboard = False
+        if self.os.is_x86():
+            usb_tablet = self._os_object.supports_usbtablet()
+        if self.os.is_arm_machvirt():
+            usb_tablet = True
+            usb_keyboard = True
+
+        if usb_tablet:
+            dev = VirtualInputDevice(self.conn)
+            dev.type = "tablet"
+            dev.bus = "usb"
+            self.add_device(dev)
+        if usb_keyboard:
+            dev = VirtualInputDevice(self.conn)
+            dev.type = "keyboard"
+            dev.bus = "usb"
+            self.add_device(dev)
 
     def add_default_console_device(self):
         if self.skip_default_console:
             return
-        if self.os.is_xenpv():
-            return
         if self.get_devices("console") or self.get_devices("serial"):
             return
 
         dev = VirtualConsoleDevice(self.conn)
         dev.type = dev.TYPE_PTY
-
-        if (self.os.is_x86() and
-            self._os_object.supports_virtioconsole() and
-            self.conn.check_support(
-            self.conn.SUPPORT_CONN_VIRTIO_CONSOLE)):
-            dev.target_type = "virtio"
-
+        if self.os.is_s390x():
+            dev.target_type = "sclp"
         self.add_device(dev)
 
     def add_default_video_device(self):
@@ -610,25 +672,43 @@ class Guest(XMLBuilder):
     def add_default_usb_controller(self):
         if self.os.is_container():
             return
-        if not self.os.is_x86():
-            return
         if any([d.type == "usb" for d in self.get_devices("controller")]):
             return
-        if not self.conn.check_support(
-            self.conn.SUPPORT_CONN_DEFAULT_USB2):
+
+        usb2 = False
+        usb3 = False
+        if self.os.is_x86():
+            usb2 = True
+        elif (self.os.is_arm_machvirt() and
+              self.conn.check_support(
+                  self.conn.SUPPORT_CONN_MACHVIRT_PCI_DEFAULT)):
+            usb3 = True
+
+
+        if not usb2 and not usb3:
             return
-        for dev in VirtualController.get_usb2_controllers(self.conn):
-            self.add_device(dev)
+
+        if usb2:
+            if not self.conn.check_support(
+                self.conn.SUPPORT_CONN_DEFAULT_USB2):
+                return
+            for dev in VirtualController.get_usb2_controllers(self.conn):
+                self.add_device(dev)
+
+        if usb3:
+            self.add_device(
+                VirtualController.get_usb3_controller(self.conn, self))
 
     def add_default_channels(self):
         if self.skip_default_channel:
             return
         if self.get_devices("channel"):
             return
+        if self.os.is_s390x():
+            # Not wanted for s390 apparently
+            return
 
-        # Skip qemu-ga on ARM where virtio slots are currently limited
         if (self.conn.is_qemu() and
-            not self.os.is_arm() and
             self._os_object.supports_qemu_ga() and
             self.conn.check_support(self.conn.SUPPORT_CONN_AUTOSOCKET)):
             dev = VirtualChannelDevice(self.conn)
@@ -642,12 +722,30 @@ class Guest(XMLBuilder):
             return
         if self.get_devices("graphics"):
             return
-        if self.os.is_container():
+        if self.os.is_container() and not self.conn.is_vz():
             return
-        if self.os.arch not in ["x86_64", "i686", "ppc64", "ppc64le", "ia64"]:
+        if self.os.arch not in ["x86_64", "i686", "ppc64", "ppc64le"]:
             return
         self.add_device(VirtualGraphics(self.conn))
 
+    def add_default_rng(self):
+        if self.skip_default_rng:
+            return
+        if self.get_devices("rng"):
+            return
+        if not (self.os.is_x86() or
+                self.os.is_arm_machvirt() or
+                self.os.is_pseries()):
+            return
+
+        if (self.conn.is_qemu() and
+            self._os_object.supports_virtiorng() and
+            self.conn.check_support(self.conn.SUPPORT_CONN_RNG_URANDOM)):
+            dev = VirtualRNGDevice(self.conn)
+            dev.type = "random"
+            dev.device = "/dev/urandom"
+            self.add_device(dev)
+
     def add_default_devices(self):
         self.add_default_graphics()
         self.add_default_video_device()
@@ -655,6 +753,7 @@ class Guest(XMLBuilder):
         self.add_default_console_device()
         self.add_default_usb_controller()
         self.add_default_channels()
+        self.add_default_rng()
 
     def _add_install_cdrom(self):
         if self._install_cdrom_device:
@@ -671,11 +770,11 @@ class Guest(XMLBuilder):
     def _remove_cdrom_install_media(self):
         for dev in self.get_devices("disk"):
             # Keep the install cdrom device around, but with no media attached.
-            # But only if we are a distro that doesn't have a multi
-            # stage install (aka not Windows)
+            # But only if we are installing windows which has a multi stage
+            # install.
             if (dev.is_cdrom() and
                 getattr(dev, "installer_media", False) and
-                not self.get_continue_inst()):
+                not self._os_object.is_windows()):
                 dev.path = None
 
     def _set_defaults(self):
@@ -696,9 +795,9 @@ class Guest(XMLBuilder):
         self._set_disk_defaults()
         self._add_implied_controllers()
         self._set_net_defaults()
-        self._set_input_defaults()
         self._set_video_defaults()
         self._set_sound_defaults()
+        self._set_panic_defaults()
 
     def _is_full_os_container(self):
         if not self.os.is_container():
@@ -716,9 +815,6 @@ class Guest(XMLBuilder):
 
         if not self.os.loader and self.os.is_hvm() and self.type == "xen":
             self.os.loader = "/usr/lib/xen/boot/hvmloader"
-        if self.os.os_type == "xen" and self.type == "xen":
-            # Use older libvirt 'linux' value for back compat
-            self.os.os_type = "linux"
         if self.os.kernel or self.os.init:
             self.os.bootorder = []
 
@@ -761,15 +857,17 @@ class Guest(XMLBuilder):
         hpet.name = "hpet"
         hpet.present = False
 
-        if (self._os_object.is_windows() and
-            self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_CLOCK) and
-            self._hv_supported()):
+        hv_clock = self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_CLOCK)
+        hv_clock_rhel = self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_CLOCK_RHEL)
+
+        if (self._os_object.is_windows() and self._hyperv_supported() and
+            (hv_clock or (self.stable_defaults() and hv_clock_rhel))):
             hyperv = self.clock.add_timer()
             hyperv.name = "hypervclock"
             hyperv.present = True
 
     def _set_emulator_defaults(self):
-        if self.os.is_xenpv():
+        if self.os.is_xenpv() or self.type == "vz":
             self.emulator = None
             return
 
@@ -802,18 +900,24 @@ class Guest(XMLBuilder):
         elif self.os.is_x86() and self.type == "kvm":
             if self.os.arch != self.conn.caps.host.cpu.arch:
                 return
-            self.cpu.set_special_mode(self.x86_cpu_default)
 
-    def _hv_supported(self):
+            self.cpu.set_special_mode(self.x86_cpu_default)
+            if self._os_object.broken_x2apic():
+                self.cpu.add_feature("x2apic", policy="disable")
+
+
+    def _hyperv_supported(self):
         if (self.os.loader_type == "pflash" and
             self.os_variant in ("win2k8r2", "win7")):
             return False
         return True
 
-    def check_defaults(self):
+    def update_defaults(self):
         # This is used only by virt-manager to reset any defaults that may have
         # changed through manual intervention via the customize wizard.
-        if not self._hv_supported():
+
+        # UEFI doesn't work with hyperv bits
+        if not self._hyperv_supported():
             self.features.hyperv_relaxed = None
             self.features.hyperv_vapic = None
             self.features.hyperv_spinlocks = None
@@ -827,7 +931,7 @@ class Guest(XMLBuilder):
             self.features.acpi = None
             self.features.apic = None
             self.features.pae = None
-            if self._is_full_os_container():
+            if self._is_full_os_container() and self.type != "vz":
                 self.features.privnet = True
             return
 
@@ -840,19 +944,28 @@ class Guest(XMLBuilder):
             default = False
 
         if self.features.acpi == "default":
-            self.features.acpi = self._os_object.supports_acpi(default)
+            if default:
+                self.features.acpi = self.capsinfo.guest.supports_acpi()
+            else:
+                self.features.acpi = False
         if self.features.apic == "default":
-            self.features.apic = self._os_object.supports_apic(default)
+            self.features.apic = self.capsinfo.guest.supports_apic()
         if self.features.pae == "default":
-            self.features.pae = self.conn.caps.supports_pae()
+            if (self.os.is_hvm() and
+                self.type == "xen" and
+                self.os.arch == "x86_64"):
+                self.features.pae = True
+            else:
+                self.features.pae = self.capsinfo.guest.supports_pae()
 
         if (self.features.vmport == "default" and
+            self.os.is_x86() and
             self.has_spice() and
             self.conn.check_support(self.conn.SUPPORT_CONN_VMPORT)):
             self.features.vmport = False
 
         if (self._os_object.is_windows() and
-            self._hv_supported() and
+            self._hyperv_supported() and
             self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_VAPIC)):
             if self.features.hyperv_relaxed is None:
                 self.features.hyperv_relaxed = True
@@ -918,6 +1031,8 @@ class Guest(XMLBuilder):
         for d in self.get_all_devices():
             if d.address.type != d.address.ADDRESS_TYPE_PCI:
                 continue
+            if None in [d.address.domain, d.address.bus, d.address.slot]:
+                continue
 
             addr = d.address
             addrstr = "%d%d%d" % (d.address.domain,
@@ -935,17 +1050,19 @@ class Guest(XMLBuilder):
             if len(devs) > 1 and 0 in devs:
                 devs[0].address.multifunction = True
 
-    def _hv_only_supports_virtio(self):
-        # Only supports virtio so we need to force it
-        return self.conn.is_qemu() and self.os.is_arm_machvirt()
-
-    def _hv_supports_virtio(self):
+    def _supports_virtio(self, os_support):
         if not self.conn.is_qemu():
             return False
 
-        if self._hv_only_supports_virtio():
+        # These _only_ support virtio so don't check the OS
+        if (self.os.is_arm_machvirt() or
+            self.os.is_s390x() or
+            self.os.is_pseries()):
             return True
 
+        if not os_support:
+            return False
+
         if self.os.is_x86():
             return True
 
@@ -977,11 +1094,9 @@ class Guest(XMLBuilder):
                 # We prefer virtio-scsi for machvirt, gets us hotplug
                 d.bus = "scsi"
             elif (d.is_disk() and
-                  (self._hv_only_supports_virtio() or
-                   (self._hv_supports_virtio() and
-                    self._os_object.supports_virtiodisk()))):
+                  self._supports_virtio(self._os_object.supports_virtiodisk())):
                 d.bus = "virtio"
-            elif self.os.is_pseries():
+            elif self.os.is_pseries() and d.is_cdrom():
                 d.bus = "scsi"
             elif self.os.is_arm():
                 d.bus = "sd"
@@ -1008,9 +1123,7 @@ class Guest(XMLBuilder):
         net_model = None
         if not self.os.is_hvm():
             net_model = None
-        elif (self._hv_only_supports_virtio() or
-              (self._hv_supports_virtio() and
-               self._os_object.supports_virtionet())):
+        elif self._supports_virtio(self._os_object.supports_virtionet()):
             net_model = "virtio"
         else:
             net_model = self._os_object.default_netmodel()
@@ -1020,19 +1133,6 @@ class Guest(XMLBuilder):
                 if not net.model:
                     net.model = net_model
 
-    def _set_input_defaults(self):
-        input_type = self._os_object.default_inputtype()
-        input_bus = self._os_object.default_inputbus()
-        if self.os.is_xenpv():
-            input_type = VirtualInputDevice.TYPE_MOUSE
-            input_bus = VirtualInputDevice.BUS_XEN
-
-        for inp in self.get_devices("input"):
-            if (inp.type == inp.TYPE_DEFAULT and
-                inp.bus  == inp.BUS_DEFAULT):
-                inp.type = input_type
-                inp.bus  = input_bus
-
     def _set_sound_defaults(self):
         if self.conn.check_support(
                 support.SUPPORT_CONN_SOUND_ICH6):
@@ -1052,8 +1152,9 @@ class Guest(XMLBuilder):
             gtype = self.default_graphics_type
             logging.debug("Using default_graphics=%s", gtype)
             if (gtype == "spice" and not
-                self.conn.check_support(
-                    self.conn.SUPPORT_CONN_GRAPHICS_SPICE)):
+                (self.conn.caps.host.cpu.arch in ["i686", "x86_64"] and
+                 self.conn.check_support(
+                     self.conn.SUPPORT_CONN_GRAPHICS_SPICE))):
                 logging.debug("spice requested but HV doesn't support it. "
                               "Using vnc.")
                 gtype = "vnc"
@@ -1073,6 +1174,10 @@ class Guest(XMLBuilder):
                 if dev.image_compression is None:
                     dev.image_compression = "off"
 
+            if (dev.type == "spice" and dev.gl and
+                not self.conn.check_support(self.conn.SUPPORT_CONN_SPICE_GL)):
+                raise ValueError(_("Host does not support spice GL"))
+
     def _add_spice_channels(self):
         if self.skip_default_channel:
             return
@@ -1091,13 +1196,21 @@ class Guest(XMLBuilder):
             return
         if self.get_devices("sound"):
             return
-        self.add_default_sound_device()
+        if not self.os.is_hvm():
+            return
+        if not (self.os.is_x86() or
+                self.os.is_arm_machvirt):
+            return
+
+        self.add_device(VirtualAudio(self.conn))
 
     def _add_spice_usbredir(self):
         if self.skip_default_usbredir:
             return
         if self.get_devices("redirdev"):
             return
+        if not self.os.is_x86():
+            return
         if not self.conn.check_support(self.conn.SUPPORT_CONN_USBREDIR):
             return
 
@@ -1115,6 +1228,11 @@ class Guest(XMLBuilder):
             if gfx.type == gfx.TYPE_SPICE:
                 return True
 
+    def has_gl(self):
+        for gfx in self.get_devices("graphics"):
+            if gfx.gl:
+                return True
+
     def _set_video_defaults(self):
         if self.has_spice():
             self._add_spice_channels()
@@ -1122,9 +1240,16 @@ class Guest(XMLBuilder):
             self._add_spice_usbredir()
 
         video_model = self._os_object.default_videomodel(self)
-        if video_model == 'vmvga' and self.stable_defaults(force=True):
-            video_model = 'vga'
+        if self.os.is_arm_machvirt():
+            video_model = "virtio"
 
         for video in self.get_devices("video"):
             if video.model == video.MODEL_DEFAULT:
                 video.model = video_model
+                if video.model == 'virtio' and self.has_gl():
+                    video.accel3d = True
+
+    def _set_panic_defaults(self):
+        for panic in self.get_devices("panic"):
+            if panic.model == VirtualPanicDevice.MODEL_DEFAULT:
+                panic.model = VirtualPanicDevice.get_default_model(self.os)
